@@ -10,7 +10,7 @@
    This file is part of LibVEX, a library for dynamic binary
    instrumentation and translation.
 
-   Copyright (C) 2004-2007 OpenWorks LLP.  All rights reserved.
+   Copyright (C) 2004-2008 OpenWorks LLP.  All rights reserved.
 
    This library is made available under a dual licensing scheme.
 
@@ -61,17 +61,19 @@
 
    Code blocks
    ~~~~~~~~~~~
-   The code is broken into small code blocks (type: 'IRBB').  Each
-   code block typically represents from 1 to perhaps 50 instructions.
-   IRBBs are single-entry, multiple-exit code blocks.  Each IRBB
-   contains three things:
+   The code is broken into small code blocks ("superblocks", type:
+   'IRSB').  Each code block typically represents from 1 to perhaps 50
+   instructions.  IRSBs are single-entry, multiple-exit code blocks.
+   Each IRSB contains three things:
    - a type environment, which indicates the type of each temporary
-     value present in the IRBB
+     value present in the IRSB
    - a list of statements, which represent code
-   - a jump that exits from the end the IRBB
+   - a jump that exits from the end the IRSB
    Because the blocks are multiple-exit, there can be additional
-   conditional exit statements that cause control to leave the IRBB
-   before the final exit.
+   conditional exit statements that cause control to leave the IRSB
+   before the final exit.  Also because of this, IRSBs can cover
+   multiple non-consecutive sequences of code (up to 3).  These are
+   recorded in the type VexGuestExtents (see libvex.h).
 
    Statements and expressions
    ~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -99,11 +101,11 @@
 
    The basic "Get" and "Put" operations are sufficient to model normal
    fixed registers on the guest.  Selected areas of the guest state
-   can be treated as a circular array of registers (type: 'IRArray'),
-   which can be indexed at run-time.  This is done with the "GetI" and
-   "PutI" primitives.  This is necessary to describe rotating register
-   files, for example the x87 FPU stack, SPARC register windows, and
-   the Itanium register files.
+   can be treated as a circular array of registers (type:
+   'IRRegArray'), which can be indexed at run-time.  This is done with
+   the "GetI" and "PutI" primitives.  This is necessary to describe
+   rotating register files, for example the x87 FPU stack, SPARC
+   register windows, and the Itanium register files.
 
    Examples, and flattened vs. unflattened code
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -176,7 +178,7 @@
 
    SSAness and typing
    ~~~~~~~~~~~~~~~~~~
-   The IR is fully typed.  For every IRBB (IR block) it is possible to
+   The IR is fully typed.  For every IRSB (IR block) it is possible to
    say unambiguously whether or not it is correctly typed.
    Incorrectly typed IR has no meaning and the VEX will refuse to
    process it.  At various points during processing VEX typechecks the
@@ -210,13 +212,15 @@
 
    - eqIRFoo is a structural equality predicate for IRFoos.
 
-   - dopyIRFoo is a deep copy constructor for IRFoos. 
+   - deepCopyIRFoo is a deep copy constructor for IRFoos. 
      It recursively traverses the entire argument tree and
-     produces a complete new tree.
+     produces a complete new tree.  All types have a deep copy
+     constructor.
 
-   - sopyIRFoo is the shallow copy constructor for IRFoos.
+   - shallowCopyIRFoo is the shallow copy constructor for IRFoos.
      It creates a new top-level copy of the supplied object,
-     but does not copy any sub-objects.
+     but does not copy any sub-objects.  Only some types have a
+     shallow copy constructor.
 */
 
 /* ------------------ Types ------------------ */
@@ -225,8 +229,8 @@
    float, or a vector (SIMD) value. */
 typedef 
    enum { 
-      Ity_INVALID=0x10FFF,
-      Ity_I1=0x11000, 
+      Ity_INVALID=0x11000,
+      Ity_I1, 
       Ity_I8, 
       Ity_I16, 
       Ity_I32, 
@@ -250,8 +254,8 @@ extern Int sizeofIRType ( IRType );
 /* IREndness is used in load IRExprs and store IRStmts. */
 typedef
    enum { 
-      Iend_LE=22, /* little endian */
-      Iend_BE=33  /* big endian */
+      Iend_LE=0x12000, /* little endian */
+      Iend_BE          /* big endian */
    }
    IREndness;
 
@@ -263,7 +267,7 @@ typedef
 /* The various kinds of constant. */
 typedef
    enum { 
-      Ico_U1=0x12000,
+      Ico_U1=0x13000,
       Ico_U8, 
       Ico_U16, 
       Ico_U32, 
@@ -307,7 +311,7 @@ extern IRConst* IRConst_F64i ( ULong );
 extern IRConst* IRConst_V128 ( UShort );
 
 /* Deep-copy an IRConst */
-extern IRConst* dopyIRConst ( IRConst* );
+extern IRConst* deepCopyIRConst ( IRConst* );
 
 /* Pretty-print an IRConst */
 extern void ppIRConst ( IRConst* );
@@ -345,7 +349,7 @@ typedef
 extern IRCallee* mkIRCallee ( Int regparms, HChar* name, void* addr );
 
 /* Deep-copy an IRCallee. */
-extern IRCallee* dopyIRCallee ( IRCallee* );
+extern IRCallee* deepCopyIRCallee ( IRCallee* );
 
 /* Pretty-print an IRCallee. */
 extern void ppIRCallee ( IRCallee* );
@@ -362,14 +366,14 @@ typedef
       IRType elemTy; /* type of each element in the indexed area */
       Int    nElems; /* number of elements in the indexed area */
    }
-   IRArray;
+   IRRegArray;
 
-extern IRArray* mkIRArray ( Int, IRType, Int );
+extern IRRegArray* mkIRRegArray ( Int, IRType, Int );
 
-extern IRArray* dopyIRArray ( IRArray* );
+extern IRRegArray* deepCopyIRRegArray ( IRRegArray* );
 
-extern void ppIRArray ( IRArray* );
-extern Bool eqIRArray ( IRArray*, IRArray* );
+extern void ppIRRegArray ( IRRegArray* );
+extern Bool eqIRRegArray ( IRRegArray*, IRRegArray* );
 
 
 /* ------------------ Temporaries ------------------ */
@@ -402,7 +406,7 @@ typedef
       /* -- Do not change this ordering.  The IR generators rely on
             (eg) Iop_Add64 == IopAdd8 + 3. -- */
 
-      Iop_INVALID=0x13000,
+      Iop_INVALID=0x14000,
       Iop_Add8,  Iop_Add16,  Iop_Add32,  Iop_Add64,
       Iop_Sub8,  Iop_Sub16,  Iop_Sub32,  Iop_Sub64,
       /* Signless mul.  MullS/MullU is elsewhere. */
@@ -418,7 +422,6 @@ typedef
       Iop_CmpNE8,  Iop_CmpNE16,  Iop_CmpNE32,  Iop_CmpNE64,
       /* Tags for unary ops */
       Iop_Not8,  Iop_Not16,  Iop_Not32,  Iop_Not64,
-      Iop_Neg8,  Iop_Neg16,  Iop_Neg32,  Iop_Neg64,
 
       /* -- Ordering not important after here. -- */
 
@@ -441,6 +444,9 @@ typedef
 
       /* As a sop to Valgrind-Memcheck, the following are useful. */
       Iop_CmpNEZ8, Iop_CmpNEZ16,  Iop_CmpNEZ32,  Iop_CmpNEZ64,
+      Iop_CmpwNEZ32, Iop_CmpwNEZ64, /* all-0s -> all-Os; other -> all-1s */
+      Iop_Left8, Iop_Left16, Iop_Left32, Iop_Left64, /*  \x -> x | -x */
+      Iop_Max32U, /* unsigned max */
 
       /* PowerPC-style 3-way integer comparisons.  Without them it is
          difficult to simulate PPC efficiently.
@@ -581,7 +587,7 @@ typedef
       /* Reinterpretation.  Take an F64 and produce an I64 with 
          the same bit pattern, or vice versa. */
       Iop_ReinterpF64asI64, Iop_ReinterpI64asF64,
-                            Iop_ReinterpI32asF32,
+      Iop_ReinterpF32asI32, Iop_ReinterpI32asF32,
 
       /* --- guest x86/amd64 specifics, not mandated by 754. --- */
 
@@ -624,6 +630,10 @@ typedef
 
       /* :: F64 -> F64 */
       Iop_Est5FRSqrt,    /* reciprocal square root estimate, 5 good bits */
+      Iop_RoundF64toF64_NEAREST, /* frin */
+      Iop_RoundF64toF64_NegINF,  /* frim */ 
+      Iop_RoundF64toF64_PosINF,  /* frip */
+      Iop_RoundF64toF64_ZERO,    /* friz */
 
       /* :: F64 -> F32 */
       Iop_TruncF64asF32, /* do F64->F32 truncation as per 'fsts' */
@@ -653,7 +663,7 @@ typedef
       Iop_QSub8Sx8, Iop_QSub16Sx4,
 
       /* MULTIPLICATION (normal / high half of signed/unsigned) */
-      Iop_Mul16x4,
+      Iop_Mul16x4, Iop_Mul32x2,
       Iop_MulHi16Ux4,
       Iop_MulHi16Sx4,
 
@@ -672,7 +682,7 @@ typedef
       Iop_CmpGT8Sx8, Iop_CmpGT16Sx4, Iop_CmpGT32Sx2,
 
       /* VECTOR x SCALAR SHIFT (shift amt :: Ity_I8) */
-                   Iop_ShlN16x4, Iop_ShlN32x2,
+      Iop_ShlN8x8, Iop_ShlN16x4, Iop_ShlN32x2,
                    Iop_ShrN16x4, Iop_ShrN32x2,
       Iop_SarN8x8, Iop_SarN16x4, Iop_SarN32x2,
 
@@ -687,6 +697,19 @@ typedef
       Iop_InterleaveHI8x8, Iop_InterleaveHI16x4, Iop_InterleaveHI32x2,
       Iop_InterleaveLO8x8, Iop_InterleaveLO16x4, Iop_InterleaveLO32x2,
 
+      /* CONCATENATION -- build a new value by concatenating either
+         the even or odd lanes of both operands.  Note that
+         Cat{Odd,Even}Lanes32x2 are identical to Interleave{HI,LO}32x2
+         and so are omitted. */
+      Iop_CatOddLanes16x4, Iop_CatEvenLanes16x4,
+
+      /* PERMUTING -- copy src bytes to dst,
+         as indexed by control vector bytes:
+            for i in 0 .. 7 . result[i] = argL[ argR[i] ] 
+         argR[i] values may only be in the range 0 .. 7, else behaviour
+         is undefined. */
+      Iop_Perm8x8,
+
       /* ------------------ 128-bit SIMD FP. ------------------ */
 
       /* --- 32x4 vector FP --- */
@@ -879,10 +902,10 @@ typedef
    in the comments for IRExpr. */
 typedef
    enum { 
-      Iex_Binder,
+      Iex_Binder=0x15000,
       Iex_Get,
       Iex_GetI,
-      Iex_Tmp,
+      Iex_RdTmp,
       Iex_Qop,
       Iex_Triop,
       Iex_Binop,
@@ -931,7 +954,7 @@ struct _IRExpr {
          example is the x87 FP register stack.
 
          The part of the guest state to be treated as a circular array
-         is described in the IRArray 'descr' field.  It holds the
+         is described in the IRRegArray 'descr' field.  It holds the
          offset of the first element in the array, the type of each
          element, and the number of elements.
 
@@ -961,9 +984,9 @@ struct _IRExpr {
                          eg. GETI(128:8xI8)[t1,0]
       */
       struct {
-         IRArray* descr;   /* Part of guest state treated as circular */
-         IRExpr*  ix;      /* Variable part of index into array */
-         Int      bias;    /* Constant offset part of index into array */
+         IRRegArray* descr; /* Part of guest state treated as circular */
+         IRExpr*     ix;    /* Variable part of index into array */
+         Int         bias;  /* Constant offset part of index into array */
       } GetI;
 
       /* The value held by a temporary.
@@ -971,7 +994,7 @@ struct _IRExpr {
       */
       struct {
          IRTemp tmp;       /* The temporary number */
-      } Tmp;
+      } RdTmp;
 
       /* A quaternary operation.
          ppIRExpr output: <op>(<arg1>, <arg2>, <arg3>, <arg4>),
@@ -1092,8 +1115,8 @@ struct _IRExpr {
 /* Expression constructors. */
 extern IRExpr* IRExpr_Binder ( Int binder );
 extern IRExpr* IRExpr_Get    ( Int off, IRType ty );
-extern IRExpr* IRExpr_GetI   ( IRArray* descr, IRExpr* ix, Int bias );
-extern IRExpr* IRExpr_Tmp    ( IRTemp tmp );
+extern IRExpr* IRExpr_GetI   ( IRRegArray* descr, IRExpr* ix, Int bias );
+extern IRExpr* IRExpr_RdTmp  ( IRTemp tmp );
 extern IRExpr* IRExpr_Qop    ( IROp op, IRExpr* arg1, IRExpr* arg2, 
                                         IRExpr* arg3, IRExpr* arg4 );
 extern IRExpr* IRExpr_Triop  ( IROp op, IRExpr* arg1, 
@@ -1106,7 +1129,7 @@ extern IRExpr* IRExpr_CCall  ( IRCallee* cee, IRType retty, IRExpr** args );
 extern IRExpr* IRExpr_Mux0X  ( IRExpr* cond, IRExpr* expr0, IRExpr* exprX );
 
 /* Deep-copy an IRExpr. */
-extern IRExpr* dopyIRExpr ( IRExpr* );
+extern IRExpr* deepCopyIRExpr ( IRExpr* );
 
 /* Pretty-print an IRExpr. */
 extern void ppIRExpr ( IRExpr* );
@@ -1118,15 +1141,19 @@ extern IRExpr** mkIRExprVec_1 ( IRExpr* );
 extern IRExpr** mkIRExprVec_2 ( IRExpr*, IRExpr* );
 extern IRExpr** mkIRExprVec_3 ( IRExpr*, IRExpr*, IRExpr* );
 extern IRExpr** mkIRExprVec_4 ( IRExpr*, IRExpr*, IRExpr*, IRExpr* );
-extern IRExpr** mkIRExprVec_5 ( IRExpr*, IRExpr*, 
+extern IRExpr** mkIRExprVec_5 ( IRExpr*, IRExpr*, IRExpr*, IRExpr*,
+                                IRExpr* );
+extern IRExpr** mkIRExprVec_6 ( IRExpr*, IRExpr*, IRExpr*, IRExpr*,
+                                IRExpr*, IRExpr* );
+extern IRExpr** mkIRExprVec_7 ( IRExpr*, IRExpr*, IRExpr*, IRExpr*,
                                 IRExpr*, IRExpr*, IRExpr* );
 
 /* IRExpr copiers:
-   - sopy: shallow-copy (ie. create a new vector that shares the
+   - shallowCopy: shallow-copy (ie. create a new vector that shares the
      elements with the original).
-   - dopy: deep-copy (ie. create a completely new vector). */
-extern IRExpr** sopyIRExprVec ( IRExpr** );
-extern IRExpr** dopyIRExprVec ( IRExpr** );
+   - deepCopy: deep-copy (ie. create a completely new vector). */
+extern IRExpr** shallowCopyIRExprVec ( IRExpr** );
+extern IRExpr** deepCopyIRExprVec ( IRExpr** );
 
 /* Make a constant expression from the given host word taking into
    account (of course) the host word size. */
@@ -1142,7 +1169,7 @@ IRExpr* mkIRExprCCall ( IRType retty,
 /* Convenience functions for atoms (IRExprs which are either Iex_Tmp or
  * Iex_Const). */
 static inline Bool isIRAtom ( IRExpr* e ) {
-   return toBool(e->tag == Iex_Tmp || e->tag == Iex_Const);
+   return toBool(e->tag == Iex_RdTmp || e->tag == Iex_Const);
 }
 
 /* Are these two IR atoms identical?  Causes an assertion
@@ -1172,7 +1199,7 @@ extern Bool eqIRAtom ( IRExpr*, IRExpr* );
 */
 typedef
    enum { 
-      Ijk_Boring=0x14000, /* not interesting; just goto next */
+      Ijk_Boring=0x16000, /* not interesting; just goto next */
       Ijk_Call,           /* guest is doing a call */
       Ijk_Ret,            /* guest is doing a return */
       Ijk_ClientReq,      /* do guest client req before continuing */
@@ -1183,12 +1210,15 @@ typedef
       Ijk_MapFail,        /* Vex-provided address translation failed */
       Ijk_TInval,         /* Invalidate translations before continuing. */
       Ijk_NoRedir,        /* Jump to un-redirected guest addr */
-      Ijk_Trap,           /* current instruction did a user trap */
+      Ijk_SigTRAP,        /* current instruction synths SIGTRAP */
+      Ijk_SigSEGV,        /* current instruction synths SIGSEGV */
       /* Unfortunately, various guest-dependent syscall kinds.  They
 	 all mean: do a syscall before continuing. */
       Ijk_Sys_syscall,    /* amd64 'syscall', ppc 'sc' */
       Ijk_Sys_int32,      /* amd64/x86 'int $0x20' */
       Ijk_Sys_int128,     /* amd64/x86 'int $0x80' */
+      Ijk_Sys_int129,     /* amd64/x86 'int $0x81' */
+      Ijk_Sys_int130,     /* amd64/x86 'int $0x82' */
       Ijk_Sys_sysenter    /* x86 'sysenter'.  guest_EIP becomes 
                              invalid at the point this happens. */
    }
@@ -1244,7 +1274,7 @@ extern void ppIRJumpKind ( IRJumpKind );
 /* Effects on resources (eg. registers, memory locations) */
 typedef
    enum {
-      Ifx_None = 0x15000,   /* no effect */
+      Ifx_None = 0x17000,   /* no effect */
       Ifx_Read,             /* reads the resource */
       Ifx_Write,            /* writes the resource */
       Ifx_Modify,           /* modifies the resource */
@@ -1286,7 +1316,7 @@ extern void     ppIRDirty ( IRDirty* );
 extern IRDirty* emptyIRDirty ( void );
 
 /* Deep-copy a dirty call */
-extern IRDirty* dopyIRDirty ( IRDirty* );
+extern IRDirty* deepCopyIRDirty ( IRDirty* );
 
 /* A handy function which takes some of the tedium out of constructing
    dirty helper calls.  The called function impliedly does not return
@@ -1306,6 +1336,21 @@ IRDirty* unsafeIRDirty_1_N ( IRTemp dst,
                              IRExpr** args );
 
 
+/* --------------- Memory Bus Events --------------- */
+
+typedef
+   enum { 
+      Imbe_Fence=0x18000, 
+      Imbe_BusLock, 
+      Imbe_BusUnlock,
+      Imbe_SnoopedStoreBegin,
+      Imbe_SnoopedStoreEnd
+   }
+   IRMBusEvent;
+
+extern void ppIRMBusEvent ( IRMBusEvent );
+
+
 /* ------------------ Statements ------------------ */
 
 /* The different kinds of statements.  Their meaning is explained
@@ -1317,17 +1362,18 @@ IRDirty* unsafeIRDirty_1_N ( IRTemp dst,
    they are required by some IR consumers such as tools that
    instrument the code.
 */
+
 typedef 
    enum {
-      Ist_NoOp,
+      Ist_NoOp=0x19000,
       Ist_IMark,     /* META */
       Ist_AbiHint,   /* META */
       Ist_Put,
       Ist_PutI,
-      Ist_Tmp,
+      Ist_WrTmp,
       Ist_Store,
       Ist_Dirty,
-      Ist_MFence,
+      Ist_MBE,       /* META (maybe) */
       Ist_Exit
    } 
    IRStmtTag;
@@ -1356,7 +1402,7 @@ typedef
          /* META: instruction mark.  Marks the start of the statements
             that represent a single machine instruction (the end of
             those statements is marked by the next IMark or the end of
-            the IRBB).  Contains the address and length of the
+            the IRSB).  Contains the address and length of the
             instruction.
 
             ppIRExpr output: ------ IMark(<addr>, <len>) ------,
@@ -1374,14 +1420,17 @@ typedef
             that a given chunk of address space, [base .. base+len-1],
             has become undefined.  This is used on amd64-linux and
             some ppc variants to pass stack-redzoning hints to whoever
-            wants to see them.
+            wants to see them.  It also indicates the address of the
+            next (dynamic) instruction that will be executed.  This is
+            to help Memcheck to origin tracking.
 
-            ppIRExpr output: ====== AbiHint(<base>, <len>) ======
-                         eg. ====== AbiHint(t1, 16) ======
+            ppIRExpr output: ====== AbiHint(<base>, <len>, <nia>) ======
+                         eg. ====== AbiHint(t1, 16, t2) ======
          */
          struct {
             IRExpr* base;     /* Start  of undefined chunk */
             Int     len;      /* Length of undefined chunk */
+            IRExpr* nia;      /* Address of next (guest) insn */
          } AbiHint;
 
          /* Write a guest register, at a fixed offset in the guest state.
@@ -1400,22 +1449,23 @@ typedef
                          eg. PUTI(64:8xF64)[t5,0] = t1
          */
          struct {
-            IRArray* descr;   /* Part of guest state treated as circular */
-            IRExpr*  ix;      /* Variable part of index into array */
-            Int      bias;    /* Constant offset part of index into array */
-            IRExpr*  data;    /* The value to write */
+            IRRegArray* descr; /* Part of guest state treated as circular */
+            IRExpr*     ix;    /* Variable part of index into array */
+            Int         bias;  /* Constant offset part of index into array */
+            IRExpr*     data;  /* The value to write */
          } PutI;
 
-         /* Assign a value to a temporary.  ("Tmp" is not a very good
-            name for this, particularly because there is also a Tmp
-            expression kind.)
+         /* Assign a value to a temporary.  Note that SSA rules require
+            each tmp is only assigned to once.  IR sanity checking will
+            reject any block containing a temporary which is not assigned
+            to exactly once.
 
             ppIRExpr output: t<tmp> = <data>, eg. t1 = 3
          */
          struct {
             IRTemp  tmp;   /* Temporary  (LHS of assignment) */
             IRExpr* data;  /* Expression (RHS of assignment) */
-         } Tmp;
+         } WrTmp;
 
          /* Write a value to memory.
             ppIRExpr output: ST<end>(<addr>) = <data>, eg. STle(t1) = t2
@@ -1441,13 +1491,17 @@ typedef
             IRDirty* details;
          } Dirty;
 
-         /* A memory fence.
-            ppIRExpr output: IR-MFence
+         /* A memory bus event - a fence, or acquisition/release of the
+            hardware bus lock.  IR optimisation treats all these as fences
+            across which no memory references may be moved.
+            ppIRExpr output: MBusEvent-Fence,
+                             MBusEvent-BusLock, MBusEvent-BusUnlock.
          */
          struct {
-         } MFence;
+            IRMBusEvent event;
+         } MBE;
 
-         /* Conditional exit from the middle of an IRBB.
+         /* Conditional exit from the middle of an IRSB.
             ppIRExpr output: if (<guard>) goto {<jk>} <dst>
                          eg. if (t69) goto {Boring} 0x4000AAA:I32
          */
@@ -1463,18 +1517,18 @@ typedef
 /* Statement constructors. */
 extern IRStmt* IRStmt_NoOp    ( void );
 extern IRStmt* IRStmt_IMark   ( Addr64 addr, Int len );
-extern IRStmt* IRStmt_AbiHint ( IRExpr* base, Int len );
+extern IRStmt* IRStmt_AbiHint ( IRExpr* base, Int len, IRExpr* nia );
 extern IRStmt* IRStmt_Put     ( Int off, IRExpr* data );
-extern IRStmt* IRStmt_PutI    ( IRArray* descr, IRExpr* ix, Int bias, 
+extern IRStmt* IRStmt_PutI    ( IRRegArray* descr, IRExpr* ix, Int bias, 
                                 IRExpr* data );
-extern IRStmt* IRStmt_Tmp     ( IRTemp tmp, IRExpr* data );
+extern IRStmt* IRStmt_WrTmp   ( IRTemp tmp, IRExpr* data );
 extern IRStmt* IRStmt_Store   ( IREndness end, IRExpr* addr, IRExpr* data );
 extern IRStmt* IRStmt_Dirty   ( IRDirty* details );
-extern IRStmt* IRStmt_MFence  ( void );
+extern IRStmt* IRStmt_MBE     ( IRMBusEvent event );
 extern IRStmt* IRStmt_Exit    ( IRExpr* guard, IRJumpKind jk, IRConst* dst );
 
 /* Deep-copy an IRStmt. */
-extern IRStmt* dopyIRStmt ( IRStmt* );
+extern IRStmt* deepCopyIRStmt ( IRStmt* );
 
 /* Pretty-print an IRStmt. */
 extern void ppIRStmt ( IRStmt* );
@@ -1500,22 +1554,24 @@ typedef
 extern IRTemp newIRTemp ( IRTypeEnv*, IRType );
 
 /* Deep-copy a type environment */
-extern IRTypeEnv* dopyIRTypeEnv ( IRTypeEnv* );
+extern IRTypeEnv* deepCopyIRTypeEnv ( IRTypeEnv* );
 
 /* Pretty-print a type environment */
 extern void ppIRTypeEnv ( IRTypeEnv* );
 
 
-/* Code blocks contain:
+/* Code blocks, which in proper compiler terminology are superblocks
+   (single entry, multiple exit code sequences) contain:
+
    - A table giving a type for each temp (the "type environment")
    - An expandable array of statements
    - An expression of type 32 or 64 bits, depending on the
-     guest's word size, indicating the next destination.
+     guest's word size, indicating the next destination if the block 
+     executes all the way to the end, without a side exit
    - An indication of any special actions (JumpKind) needed
      for this final jump.
    
-   The "BB" is short for "basic block", but this is a misnomer, as an
-   IRBB can contain multiple exits.
+   "IRSB" stands for "IR Super Block".
 */
 typedef
    struct {
@@ -1526,22 +1582,23 @@ typedef
       IRExpr*    next;
       IRJumpKind jumpkind;
    }
-   IRBB;
+   IRSB;
 
-/* Allocate a new, uninitialised IRBB */
-extern IRBB* emptyIRBB ( void );
+/* Allocate a new, uninitialised IRSB */
+extern IRSB* emptyIRSB ( void );
 
-/* Deep-copy an IRBB */
-extern IRBB* dopyIRBB ( IRBB* );
+/* Deep-copy an IRSB */
+extern IRSB* deepCopyIRSB ( IRSB* );
 
-/* Deep-copy an IRBB, except for the statements list. */
-extern IRBB* dopyIRBBExceptStmts ( IRBB* );
+/* Deep-copy an IRSB, except for the statements list, which set to be
+   a new, empty, list of statements. */
+extern IRSB* deepCopyIRSBExceptStmts ( IRSB* );
 
-/* Pretty-print an IRBB */
-extern void ppIRBB ( IRBB* );
+/* Pretty-print an IRSB */
+extern void ppIRSB ( IRSB* );
 
-/* Append an IRStmt to an IRBB */
-extern void addStmtToIRBB ( IRBB*, IRStmt* );
+/* Append an IRStmt to an IRSB */
+extern void addStmtToIRSB ( IRSB*, IRStmt* );
 
 
 /*---------------------------------------------------------------*/
@@ -1557,7 +1614,7 @@ extern IRType typeOfIRTemp  ( IRTypeEnv*, IRTemp );
 extern IRType typeOfIRExpr  ( IRTypeEnv*, IRExpr* );
 
 /* Sanity check a BB of IR */
-extern void sanityCheckIRBB ( IRBB*  bb, 
+extern void sanityCheckIRSB ( IRSB*  bb, 
                               HChar* caller,
                               Bool   require_flatness, 
                               IRType guest_word_size );
