@@ -8,7 +8,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward 
+   Copyright (C) 2000-2008 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -46,6 +46,8 @@
 
 #include "pub_core_basics.h"
 #include "pub_core_libcassert.h"    // For VG_BUGS_TO
+#include "pub_core_vki.h"           // Avoids warnings from 
+                                    // pub_core_libcfile.h
 #include "pub_core_libcfile.h"      // For VG_CLO_DEFAULT_LOGPORT
 
 
@@ -126,7 +128,8 @@ static void copyout ( char* buf, int nbuf )
       if (buf[i] == '\n') {
          fprintf(stdout, "\n(%d) ", conn_count);
       } else {
-         fwrite(&buf[i], 1, 1, stdout);
+         __attribute__((unused)) size_t ignored 
+            = fwrite(&buf[i], 1, 1, stdout);
       }
    }
    fflush(stdout);
@@ -228,7 +231,8 @@ static void sigint_handler ( int signo )
 int main (int argc, char** argv) 
 {
    int    i, j, k, res, one;
-   int    main_sd, new_sd, client_len;
+   int    main_sd, new_sd;
+   socklen_t client_len;
    struct sockaddr_in client_addr, server_addr;
 
    char /*bool*/ exit_when_zero = 0;
@@ -304,7 +308,7 @@ int main (int argc, char** argv)
 
            /* ok, we have someone waiting to connect.  Get the sd. */
            client_len = sizeof(client_addr);
-           new_sd = accept(main_sd, (struct sockaddr *) &client_addr, 
+           new_sd = accept(main_sd, (struct sockaddr *)&client_addr, 
                                                        &client_len);
            if (new_sd < 0) {
               perror("cannot accept connection ");
