@@ -7,7 +7,7 @@
    This file is part of Cachegrind, a Valgrind tool for cache
    profiling programs.
 
-   Copyright (C) 2002-2007 Nicholas Nethercote
+   Copyright (C) 2002-2008 Nicholas Nethercote
       njn@valgrind.org
 
    This program is free software; you can redistribute it and/or
@@ -52,11 +52,16 @@ static void micro_ops_warn(Int actual_size, Int used_size, Int line_size)
 
 /* Intel method is truly wretched.  We have to do an insane indexing into an
  * array of pre-defined configurations for various parts of the memory
- * hierarchy. 
+ * hierarchy.
+ * According to Intel Processor Identification, App Note 485.
  */
 static
 Int Intel_cache_info(Int level, cache_t* I1c, cache_t* D1c, cache_t* L2c)
 {
+   Int cpuid1_eax;
+   Int cpuid1_ignore;
+   Int family;
+   Int model;
    UChar info[16];
    Int   i, trials;
    Bool  L2_found = False;
@@ -68,6 +73,12 @@ Int Intel_cache_info(Int level, cache_t* I1c, cache_t* D1c, cache_t* L2c)
       return -1;
    }
 
+   /* family/model needed to distinguish code reuse (currently 0x49) */
+   VG_(cpuid)(1, &cpuid1_eax, &cpuid1_ignore,
+	      &cpuid1_ignore, &cpuid1_ignore);
+   family = (((cpuid1_eax >> 20) & 0xff) << 4) + ((cpuid1_eax >> 8) & 0xf);
+   model =  (((cpuid1_eax >> 16) & 0xf) << 4) + ((cpuid1_eax >> 4) & 0xf);
+
    VG_(cpuid)(2, (Int*)&info[0], (Int*)&info[4], 
                  (Int*)&info[8], (Int*)&info[12]);
    trials  = info[0] - 1;   /* AL register - bits 0..7 of %eax */
@@ -89,9 +100,11 @@ Int Intel_cache_info(Int level, cache_t* I1c, cache_t* D1c, cache_t* L2c)
           
       /* TLB info, ignore */
       case 0x01: case 0x02: case 0x03: case 0x04: case 0x05:
-      case 0x50: case 0x51: case 0x52: case 0x56: case 0x57:
+      case 0x4f: case 0x50: case 0x51: case 0x52:
+      case 0x56: case 0x57: case 0x59:
       case 0x5b: case 0x5c: case 0x5d:
-      case 0xb0: case 0xb1: case 0xb3: case 0xb4:
+      case 0xb0: case 0xb1:
+      case 0xb3: case 0xb4: case 0xba: case 0xc0:
           break;      
 
       case 0x06: *I1c = (cache_t) {  8, 4, 32 }; break;
@@ -100,6 +113,12 @@ Int Intel_cache_info(Int level, cache_t* I1c, cache_t* D1c, cache_t* L2c)
 
       case 0x0a: *D1c = (cache_t) {  8, 2, 32 }; break;
       case 0x0c: *D1c = (cache_t) { 16, 4, 32 }; break;
+      case 0x0e:
+         /* Real D1 cache configuration is:
+            D1c = (cache_t) { 24, 6, 64 }; */
+         VG_(message)(Vg_DebugMsg, "warning: 24Kb D1 cache detected, treating as 16Kb");
+         *D1c = (cache_t) { 16, 4, 64 };
+         break;
       case 0x2c: *D1c = (cache_t) { 32, 8, 64 }; break;
 
       /* IA-64 info -- panic! */
@@ -108,8 +127,10 @@ Int Intel_cache_info(Int level, cache_t* I1c, cache_t* D1c, cache_t* L2c)
       case 0x90: case 0x96: case 0x9b:
          VG_(tool_panic)("IA-64 cache detected?!");
 
-      case 0x22: case 0x23: case 0x25: case 0x29: case 0x46: case 0x47:
-          VG_(message)(Vg_DebugMsg, "warning: L3 cache detected but ignored");
+      case 0x22: case 0x23: case 0x25: case 0x29:
+      case 0x46: case 0x47: case 0x4a: case 0x4b: case 0x4c: case 0x4d:
+          VG_(message)(Vg_DebugMsg,
+             "warning: L3 cache detected but ignored");
           break;
 
       /* These are sectored, whatever that means */
@@ -128,7 +149,26 @@ Int Intel_cache_info(Int level, cache_t* I1c, cache_t* D1c, cache_t* L2c)
       case 0x43: *L2c = (cache_t) {  512, 4, 32 }; L2_found = True; break;
       case 0x44: *L2c = (cache_t) { 1024, 4, 32 }; L2_found = True; break;
       case 0x45: *L2c = (cache_t) { 2048, 4, 32 }; L2_found = True; break;
-      case 0x49: *L2c = (cache_t) { 4096,16, 64 }; L2_found = True; break;
+      case 0x48:
+         /* Real L2 cache configuration is:
+            *L2c = (cache_t) { 3072, 12, 64 }; L2_found = True; */
+         VG_(message)(Vg_DebugMsg, "warning: 3Mb L2 cache detected, treating as 2Mb");
+         *L2c = (cache_t) { 2048, 8, 64 }; L2_found = True;
+         break;
+      case 0x49:
+	  if ((family == 15) && (model == 6))
+	      /* On Xeon MP (family F, model 6), this is for L3 */
+	      VG_(message)(Vg_DebugMsg, 
+			   "warning: L3 cache detected but ignored");
+	  else
+	      *L2c = (cache_t) { 4096, 16, 64 }; L2_found = True;
+	  break;
+      case 0x4e:
+         /* Real L2 cache configuration is:
+            *L2c = (cache_t) { 6144, 24, 64 }; L2_found = True; */
+         VG_(message)(Vg_DebugMsg, "warning: 6Mb L2 cache detected, treating as 4Mb");
+         *L2c = (cache_t) { 4096, 16, 64 }; L2_found = True;
+         break;
 
       /* These are sectored, whatever that means */
       case 0x60: *D1c = (cache_t) { 16, 8, 64 };  break;      /* sectored */
@@ -162,6 +202,9 @@ Int Intel_cache_info(Int level, cache_t* I1c, cache_t* D1c, cache_t* L2c)
       case 0x7d: *L2c = (cache_t) { 2048, 8,  64 }; L2_found = True;  break;
       case 0x7e: *L2c = (cache_t) {  256, 8, 128 }; L2_found = True;  break;
 
+      case 0x7f: *L2c = (cache_t) {  512, 2, 64 };  L2_found = True;  break;
+      case 0x80: *L2c = (cache_t) {  512, 8, 64 };  L2_found = True;  break;
+
       case 0x81: *L2c = (cache_t) {  128, 8, 32 };  L2_found = True;  break;
       case 0x82: *L2c = (cache_t) {  256, 8, 32 };  L2_found = True;  break;
       case 0x83: *L2c = (cache_t) {  512, 8, 32 };  L2_found = True;  break;
@@ -272,7 +315,7 @@ Int get_caches_from_CPUID(cache_t* I1c, cache_t* D1c, cache_t* L2c)
    vendor_id[12] = '\0';
 
    if (0 == level) {
-      VG_(message)(Vg_DebugMsg, "CPUID level is 0, early Pentium?\n");
+      VG_(message)(Vg_DebugMsg, "CPUID level is 0, early Pentium?");
       return -1;
    }
 
