@@ -6,7 +6,7 @@
 /*
    This file is part of Callgrind, a Valgrind tool for call tracing.
 
-   Copyright (C) 2002-2006, Josef Weidendorfer (Josef.Weidendorfer@gmx.de)
+   Copyright (C) 2002-2008, Josef Weidendorfer (Josef.Weidendorfer@gmx.de)
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -32,15 +32,13 @@
 #include <pub_tool_threadstate.h>
 #include <pub_tool_libcfile.h>
 
-/*------------------------------------------------------------*/
-/*--- Support for signal handlers and multi-threading      ---*/
-/*------------------------------------------------------------*/
 
 /* Dump Part Counter */
 static Int out_counter = 0;
 
-static Char* dump_file_base = 0;
-static Char* base_directory = 0;
+static Char* out_file = 0;
+static Char* out_directory = 0;
+static Bool dumps_initialized = False;
 
 /* Command */
 static Char cmdbuf[BUF_LEN];
@@ -64,9 +62,16 @@ Int CLG_(get_dump_counter)(void)
   return out_counter;
 }
 
-Char* CLG_(get_dump_file_base)()
+Char* CLG_(get_out_file)()
 {
-  return dump_file_base;
+    CLG_(init_dumps)();
+    return out_file;
+}
+
+Char* CLG_(get_out_directory)()
+{
+    CLG_(init_dumps)();
+    return out_directory;
 }
 
 /*------------------------------------------------------------*/
@@ -100,7 +105,8 @@ void init_dump_array(void)
       CLG_(stat).distinct_fns +
       CLG_(stat).context_counter;
     CLG_ASSERT(dump_array == 0);
-    dump_array = (Bool*) CLG_MALLOC(dump_array_size * sizeof(Bool));
+    dump_array = (Bool*) CLG_MALLOC("cl.dump.ida.1",
+                                    dump_array_size * sizeof(Bool));
     obj_dumped  = dump_array;
     file_dumped = obj_dumped + CLG_(stat).distinct_objs;
     fn_dumped   = file_dumped + CLG_(stat).distinct_files;
@@ -429,31 +435,40 @@ void init_debug_cache(void)
     }
 }
 
-static __inline__
+static /* __inline__ */
 Bool get_debug_pos(BBCC* bbcc, Addr addr, AddrPos* p)
 {
     Char file[FILENAME_LEN];
-    Bool res;
+    Char dir[FILENAME_LEN];
+    Bool found_file_line, found_dirname;
 
     int cachepos = addr % DEBUG_CACHE_SIZE;
     
     if (debug_cache_addr[cachepos] == addr) {
 	p->line = debug_cache_line[cachepos];
 	p->file = debug_cache_file[cachepos];
-	res     = debug_cache_info[cachepos];
+	found_file_line = debug_cache_info[cachepos];
     }
     else {
-	res = VG_(get_filename_linenum)(addr,
-					file, FILENAME_LEN,
-					NULL, 0, NULL, //FIXME
-					&(p->line));
-	if (!res) {
+	found_file_line = VG_(get_filename_linenum)(addr,
+						    file, FILENAME_LEN,
+						    dir, FILENAME_LEN,
+						    &found_dirname,
+						    &(p->line));
+	if (!found_file_line) {
 	    VG_(strcpy)(file, "???");
 	    p->line = 0;
 	}
+	if (found_dirname) {
+	    // +1 for the '/'.
+	    CLG_ASSERT(VG_(strlen)(dir) + VG_(strlen)(file) + 1 < FILENAME_LEN);
+	    VG_(strcat)(dir, "/");     // Append '/'
+	    VG_(strcat)(dir, file);    // Append file to dir
+	    VG_(strcpy)(file, dir);    // Move dir+file to file
+	}
 	p->file    = CLG_(get_file_node)(bbcc->bb->obj, file);
 
-	debug_cache_info[cachepos] = res;
+	debug_cache_info[cachepos] = found_file_line;
 	debug_cache_addr[cachepos] = addr;
 	debug_cache_line[cachepos] = p->line;
 	debug_cache_file[cachepos] = p->file;
@@ -463,11 +478,11 @@ Bool get_debug_pos(BBCC* bbcc, Addr addr, AddrPos* p)
     p->addr = addr - bbcc->bb->obj->offset;
     p->bb_addr = bbcc->bb->offset;
 
-    CLG_DEBUG(3, "  get_debug_pos(%p): BB %p, fn '%s', file '%s', line %u\n",
+    CLG_DEBUG(3, "  get_debug_pos(%#lx): BB %#lx, fn '%s', file '%s', line %u\n",
 	     addr, bb_addr(bbcc->bb), bbcc->cxt->fn[0]->name,
 	     p->file->name, p->line);
 
-    return res;
+    return found_file_line;
 }
 
 
@@ -505,7 +520,7 @@ static void init_fcost(AddrCost* c, Addr addr, Addr bbaddr, file_node* file)
 static void fprint_apos(Int fd, AddrPos* curr, AddrPos* last, file_node* func_file)
 {
     CLG_ASSERT(curr->file != 0);
-    CLG_DEBUG(2, "    print_apos(file '%s', line %d, bb %p, addr %p) fnFile '%s'\n",
+    CLG_DEBUG(2, "    print_apos(file '%s', line %d, bb %#lx, addr %#lx) fnFile '%s'\n",
 	     curr->file->name, curr->line, curr->bb_addr, curr->addr,
 	     func_file->name);
 
@@ -540,7 +555,7 @@ static
 void fprint_pos(Int fd, AddrPos* curr, AddrPos* last)
 {
     if (0) //CLG_(clo).dump_bbs)
-	VG_(sprintf)(outbuf, "%u ", curr->addr - curr->bb_addr);
+	VG_(sprintf)(outbuf, "%lu ", curr->addr - curr->bb_addr);
     else {
 	int p = 0;
 	if (CLG_(clo).dump_instr) {
@@ -555,7 +570,7 @@ void fprint_pos(Int fd, AddrPos* curr, AddrPos* last)
 		    p = VG_(sprintf)(outbuf, "%d ", diff);
 	    }
 	    else
-		p = VG_(sprintf)(outbuf, "%p ", curr->addr);
+		p = VG_(sprintf)(outbuf, "%#lx ", curr->addr);
 	}
 
 	if (CLG_(clo).dump_bb) {
@@ -570,7 +585,7 @@ void fprint_pos(Int fd, AddrPos* curr, AddrPos* last)
 		    p += VG_(sprintf)(outbuf+p, "%d ", diff);
 	    }
 	    else
-		p += VG_(sprintf)(outbuf+p, "%p ", curr->bb_addr);
+		p += VG_(sprintf)(outbuf+p, "%#lx ", curr->bb_addr);
 	}
 
 	if (CLG_(clo).dump_line) {
@@ -616,7 +631,7 @@ void fprint_cost(int fd, EventMapping* es, ULong* cost)
 static void fprint_fcost(Int fd, AddrCost* c, AddrPos* last)
 {
   CLG_DEBUGIF(3) {
-    CLG_DEBUG(2, "   print_fcost(file '%s', line %d, bb %p, addr %p):\n",
+    CLG_DEBUG(2, "   print_fcost(file '%s', line %d, bb %#lx, addr %#lx):\n",
 	     c->p.file->name, c->p.line, c->p.bb_addr, c->p.addr);
     CLG_(print_cost)(-5, CLG_(sets).full, c->cost);
   }
@@ -1004,7 +1019,7 @@ static void qsort(BBCC **a, int n, int (*cmp)(BBCC**,BBCC**))
 	int s, r;
 	BBCC* v;
 
-	CLG_DEBUG(8, "  qsort(%d,%d)\n", a-qsort_start, n);
+	CLG_DEBUG(8, "  qsort(%ld,%ld)\n", a-qsort_start + 0L, n + 0L);
 
 	if (n < 7) {	 /* Insertion sort on smallest arrays */
 		for (pm = a+1; pm < a+n; pm++)
@@ -1013,7 +1028,8 @@ static void qsort(BBCC **a, int n, int (*cmp)(BBCC**,BBCC**))
 
 		CLG_DEBUGIF(8) {
 		    for (pm = a; pm < a+n; pm++) {
-			VG_(printf)("   %3d BB %p, ", pm - qsort_start,
+			VG_(printf)("   %3ld BB %#lx, ",
+                                    pm - qsort_start + 0L,
 				    bb_addr((*pm)->bb));      
 			CLG_(print_cxt)(9, (*pm)->cxt, (*pm)->rec_index);
 		    }
@@ -1067,25 +1083,30 @@ static void qsort(BBCC **a, int n, int (*cmp)(BBCC**,BBCC**))
 	if ((s = a+n-1-pd)>0) { for(r=0;r<s;r++) swap(pc+r, a+n-s+r); }	    
 
 	CLG_DEBUGIF(8) {
-	  VG_(printf)("   PV BB %p, ", bb_addr((*pv)->bb));
+	  VG_(printf)("   PV BB %#lx, ", bb_addr((*pv)->bb));
 	    CLG_(print_cxt)(9, (*pv)->cxt, (*pv)->rec_index);
 
 	    s = pb-pa+1;
-	    VG_(printf)("    Lower %d - %d:\n", a-qsort_start, a+s-1-qsort_start);
+	    VG_(printf)("    Lower %ld - %ld:\n",
+                        a-qsort_start + 0L,
+                        a+s-1-qsort_start + 0L);
 	    for (r=0;r<s;r++) {
 		pm = a+r;
-		VG_(printf)("     %3d BB %p, ", 
-			    pm-qsort_start,bb_addr((*pm)->bb));
+		VG_(printf)("     %3ld BB %#lx, ",
+			    pm-qsort_start + 0L,
+                            bb_addr((*pm)->bb));
 		CLG_(print_cxt)(9, (*pm)->cxt, (*pm)->rec_index);
 	    }
 
 	    s = pd-pc+1;
-	    VG_(printf)("    Upper %d - %d:\n", 
-			a+n-s-qsort_start, a+n-1-qsort_start);
+	    VG_(printf)("    Upper %ld - %ld:\n",
+			a+n-s-qsort_start + 0L,
+                        a+n-1-qsort_start + 0L);
 	    for (r=0;r<s;r++) {
 		pm = a+n-s+r;
-		VG_(printf)("     %3d BB %p, ", 
-			    pm-qsort_start,bb_addr((*pm)->bb));
+		VG_(printf)("     %3ld BB %#lx, ",
+			    pm-qsort_start + 0L,
+                            bb_addr((*pm)->bb));
 		CLG_(print_cxt)(9, (*pm)->cxt, (*pm)->rec_index);
 	    }
 	}
@@ -1198,7 +1219,8 @@ BBCC** prepare_dump(void)
 
     /* allocate bbcc array, insert BBCCs and sort */
     prepare_ptr = array =
-      (BBCC**) CLG_MALLOC((prepare_count+1) * sizeof(BBCC*));    
+      (BBCC**) CLG_MALLOC("cl.dump.pd.1",
+                          (prepare_count+1) * sizeof(BBCC*));    
 
     CLG_(forall_bbccs)(hash_addPtr);
 
@@ -1264,10 +1286,11 @@ static int new_dumpfile(Char buf[BUF_LEN], int tid, Char* trigger)
     FullCost sum = 0;
     SysRes res;
 
+    CLG_ASSERT(dumps_initialized);
     CLG_ASSERT(filename != 0);
 
     if (!CLG_(clo).combine_dumps) {
-	i = VG_(sprintf)(filename, "%s.%d", dump_file_base, VG_(getpid)());
+	i = VG_(sprintf)(filename, "%s", out_file);
     
 	if (trigger)
 	    i += VG_(sprintf)(filename+i, ".%d", out_counter);
@@ -1278,7 +1301,7 @@ static int new_dumpfile(Char buf[BUF_LEN], int tid, Char* trigger)
 	res = VG_(open)(filename, VKI_O_WRONLY|VKI_O_TRUNC, 0);
     }
     else {
-	VG_(sprintf)(filename, "%s.%d", dump_file_base, VG_(getpid)());
+	VG_(sprintf)(filename, "%s", out_file);
         res = VG_(open)(filename, VKI_O_WRONLY|VKI_O_APPEND, 0);
 	if (!res.isError && out_counter>1)
 	    appending = True;
@@ -1293,7 +1316,7 @@ static int new_dumpfile(Char buf[BUF_LEN], int tid, Char* trigger)
 	    file_err();
 	}
     }
-    fd = (Int) res.val;
+    fd = (Int) res.res;
 
     CLG_DEBUG(2, "  new_dumpfile '%s'\n", filename);
 
@@ -1534,7 +1557,7 @@ static void print_bbccs_of_thread(thread_info* ti)
 	/* FIXME: Specify Object of BB if different to object of fn */
 	int i, pos = 0;
 	ULong ecounter = (*p)->ecounter_sum;
-	pos = VG_(sprintf)(print_buf, "bb=%p ", (*p)->bb->offset);
+	pos = VG_(sprintf)(print_buf, "bb=%#lx ", (*p)->bb->offset);
 	for(i = 0; i<(*p)->bb->cjmp_count;i++) {
 	    pos += VG_(sprintf)(print_buf+pos, "%d %llu ", 
 				(*p)->bb->jmp[i].instr,
@@ -1551,7 +1574,7 @@ static void print_bbccs_of_thread(thread_info* ti)
     
     p++;
   }
-  
+
   close_dumpfile(print_buf, print_fd, CLG_(current_tid));
   if (array) VG_(free)(array);
   
@@ -1593,6 +1616,8 @@ void CLG_(dump_profile)(Char* trigger, Bool only_current_thread)
    CLG_DEBUG(2, "+ dump_profile(Trigger '%s')\n",
 	    trigger ? trigger : (Char*)"Prg.Term.");
 
+   CLG_(init_dumps)();
+
    if (VG_(clo_verbosity) > 1)
        VG_(message)(Vg_DebugMsg, "Start dumping at BB %llu (%s)...",
 		    CLG_(stat).bb_executions,
@@ -1602,7 +1627,6 @@ void CLG_(dump_profile)(Char* trigger, Bool only_current_thread)
 
    print_bbccs(trigger, only_current_thread);
 
-
    bbs_done = CLG_(stat).bb_executions++;
 
    if (VG_(clo_verbosity) > 1)
@@ -1620,8 +1644,8 @@ void init_cmdbuf(void)
   if (VG_(args_the_exename))
       size = VG_(sprintf)(cmdbuf, " %s", VG_(args_the_exename));
 
-  for(i = 0; i < VG_(args_for_client).used; i++) {
-      argv = VG_(args_for_client).strs[i];
+  for(i = 0; i < VG_(sizeXA)( VG_(args_for_client) ); i++) {
+      argv = * (HChar**) VG_(indexXA)( VG_(args_for_client), i );
       if (!argv) continue;
       if ((size>0) && (size < BUF_LEN)) cmdbuf[size++] = ' ';
       for(j=0;argv[j]!=0;j++)
@@ -1629,7 +1653,7 @@ void init_cmdbuf(void)
   }
 #else
   for(i = 0; i < VG_(client_argc); i++) {
-    argv = VG_(client_argv[i]);
+    argv = VG_(client_argv)[i];
     if (!argv) continue;
     if ((size>0) && (size < BUF_LEN)) cmdbuf[size++] = ' ';
     for(j=0;argv[j]!=0;j++)
@@ -1641,55 +1665,65 @@ void init_cmdbuf(void)
   cmdbuf[size] = 0;
 }
 
-void CLG_(init_files)(Char** dir, Char** file)
+/*
+ * Set up file names for dump output: <out_directory>, <out_file>.
+ * <out_file> is derived from the output format string, which defaults
+ * to "callgrind.out.%p", where %p is replaced with the PID.
+ * For the final file name, on intermediate dumps a counter is appended,
+ * and further, if separate dumps per thread are requested, the thread ID.
+ *
+ * <out_file> always starts with a full absolute path.
+ * If the output format string represents a relative path, the current
+ * working directory at program start is used.
+ *
+ * This function has to be called every time a profile dump is generated
+ * to be able to react on PID changes.
+ */
+void CLG_(init_dumps)()
 {
-  Int size;
-  SysRes res;
+   Int lastSlash, i;
+   SysRes res;
 
-   if (!CLG_(clo).filename_base)
-     CLG_(clo).filename_base = DEFAULT_DUMPNAME;
+   static int thisPID = 0;
+   int currentPID = VG_(getpid)();
+   if (currentPID == thisPID) {
+       /* already initialized, and no PID change */
+       CLG_ASSERT(out_file != 0);
+       return;
+   }
+   thisPID = currentPID;
+   
+   if (!CLG_(clo).out_format)
+     CLG_(clo).out_format = DEFAULT_OUTFORMAT;
+
+   /* If a file name was already set, clean up before */
+   if (out_file) {
+       VG_(free)(out_file);
+       VG_(free)(out_directory);
+       VG_(free)(filename);
+       out_counter = 0;
+   }
+
+   // Setup output filename.
+   out_file =
+       VG_(expand_file_name)("--callgrind-out-file", CLG_(clo).out_format);
 
    /* get base directory for dump/command/result files */
-   if (CLG_(clo).filename_base[0] == '/') {
-       int lastSlash = 0, i =1;
-       while(CLG_(clo).filename_base[i]) {
-	 for(; CLG_(clo).filename_base[i] &&
-	       CLG_(clo).filename_base[i] != '/'; i++);
-	   if (CLG_(clo).filename_base[i] != '/') break;
-	   lastSlash = i;
-	   i++;
-       }
-       i = lastSlash;
-       base_directory = (Char*) CLG_MALLOC(i+1);
-       VG_(strncpy)(base_directory, CLG_(clo).filename_base, i);
-       base_directory[i] = 0;
-
-       dump_file_base = CLG_(clo).filename_base;
-   }
-   else {
-       size = 100;
-       base_directory = 0;
-
-       /* getcwd() fails if the buffer isn't big enough -- keep doubling size
-          until it succeeds. */
-       while (NULL == base_directory) {
-           base_directory = CLG_MALLOC(size);
-           if (!VG_(getcwd)(base_directory, size)) {
-               VG_(free)(base_directory);
-               base_directory = 0;
-               size *= 2;
-           }
-       }
-
-       size = VG_(strlen)(base_directory) + VG_(strlen)(CLG_(clo).filename_base) +2;
-       dump_file_base = (Char*) CLG_MALLOC(size);
-       CLG_ASSERT(dump_file_base != 0);
-       VG_(sprintf)(dump_file_base, "%s/%s",
-		    base_directory, CLG_(clo).filename_base);
+   CLG_ASSERT(out_file[0] == '/');
+   lastSlash = 0;
+   i = 1;
+   while(out_file[i]) {
+       if (out_file[i] == '/') lastSlash = i;
+       i++;
    }
+   i = lastSlash;
+   out_directory = (Char*) CLG_MALLOC("cl.dump.init_dumps.1", i+1);
+   VG_(strncpy)(out_directory, out_file, i);
+   out_directory[i] = 0;
 
    /* allocate space big enough for final filenames */
-   filename = (Char*) CLG_MALLOC(VG_(strlen)(dump_file_base)+32);
+   filename = (Char*) CLG_MALLOC("cl.dump.init_dumps.2",
+                                 VG_(strlen)(out_file)+32);
    CLG_ASSERT(filename != 0);
        
    /* Make sure the output base file can be written.
@@ -1698,7 +1732,7 @@ void CLG_(init_files)(Char** dir, Char** file)
     * file: This is probably because of missing rights,
     * and trace parts wouldn't be allowed to be written, too.
     */ 
-    VG_(sprintf)(filename, "%s.%d", dump_file_base, VG_(getpid)());
+    VG_(strcpy)(filename, out_file);
     res = VG_(open)(filename, VKI_O_WRONLY|VKI_O_TRUNC, 0);
     if (res.isError) { 
 	res = VG_(open)(filename, VKI_O_CREAT|VKI_O_WRONLY,
@@ -1707,10 +1741,10 @@ void CLG_(init_files)(Char** dir, Char** file)
 	    file_err(); 
 	}
     }
-    if (!res.isError) VG_(close)( (Int)res.val );
+    if (!res.isError) VG_(close)( (Int)res.res );
 
-    *dir  = base_directory;
-    *file = filename;
+    if (!dumps_initialized)
+	init_cmdbuf();
 
-    init_cmdbuf();
+    dumps_initialized = True;
 }
