@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward 
+   Copyright (C) 2000-2008 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -57,7 +57,7 @@ static HChar* read_dot_valgrindrc ( HChar* dir )
 {
    Int    n;
    SysRes fd;
-   Int    size;
+   struct vg_stat stat_buf;
    HChar* f_clo = NULL;
    HChar  filename[VKI_PATH_MAX];
 
@@ -65,15 +65,24 @@ static HChar* read_dot_valgrindrc ( HChar* dir )
                            ( NULL == dir ? "" : dir ) );
    fd = VG_(open)(filename, 0, VKI_S_IRUSR);
    if ( !fd.isError ) {
-      size = VG_(fsize)(fd.res);
-      if (size > 0) {
-         f_clo = VG_(malloc)(size+1);
-         vg_assert(f_clo);
-         n = VG_(read)(fd.res, f_clo, size);
-         if (n == -1) n = 0;
-         vg_assert(n >= 0 && n <= size+1);
-         f_clo[n] = '\0';
+      Int res = VG_(fstat)( fd.res, &stat_buf );
+      // Ignore if not owned by current user or world writeable (CVE-2008-4865)
+      if (!res && stat_buf.st_uid == VG_(geteuid)()
+          && (!(stat_buf.st_mode & VKI_S_IWOTH))) {
+         if ( stat_buf.st_size > 0 ) {
+            f_clo = VG_(malloc)("commandline.rdv.1", stat_buf.st_size+1);
+            vg_assert(f_clo);
+            n = VG_(read)(fd.res, f_clo, stat_buf.st_size);
+            if (n == -1) n = 0;
+            vg_assert(n >= 0 && n <= stat_buf.st_size+1);
+            f_clo[n] = '\0';
+         }
       }
+      else
+         VG_(message)(Vg_UserMsg,
+               "%s was not read as it is world writeable or not owned by the "
+               "current user", filename);
+
       VG_(close)(fd.res);
    }
    return f_clo;
@@ -154,17 +163,20 @@ void VG_(split_up_argv)( Int argc, HChar** argv )
    vg_assert(!already_called);
    already_called = True;
 
-   tmp_xarray = VG_(newXA)( VG_(malloc), VG_(free), sizeof(HChar*) );
+   tmp_xarray = VG_(newXA)( VG_(malloc), "commandline.sua.1",
+                            VG_(free), sizeof(HChar*) );
    vg_assert(tmp_xarray);
 
    vg_assert( ! VG_(args_for_valgrind) );
    VG_(args_for_valgrind)
-      = VG_(newXA)( VG_(malloc), VG_(free), sizeof(HChar*) );
+      = VG_(newXA)( VG_(malloc), "commandline.sua.2",
+                    VG_(free), sizeof(HChar*) );
    vg_assert( VG_(args_for_valgrind) );
 
    vg_assert( ! VG_(args_for_client) );
    VG_(args_for_client)
-      = VG_(newXA)( VG_(malloc), VG_(free), sizeof(HChar*) );
+      = VG_(newXA)( VG_(malloc), "commandline.sua.3",
+                    VG_(free), sizeof(HChar*) );
    vg_assert( VG_(args_for_client) );
 
    /* Collect up the args-for-V. */
@@ -203,7 +215,8 @@ void VG_(split_up_argv)( Int argc, HChar** argv )
       // put into VG_(args_for_valgrind) and so must persist.
       HChar* home    = VG_(getenv)("HOME");
       HChar* f1_clo  = home ? read_dot_valgrindrc( home ) : NULL;
-      HChar* env_clo = VG_(strdup)( VG_(getenv)(VALGRIND_OPTS) );
+      HChar* env_clo = VG_(strdup)( "commandline.sua.4",
+                                    VG_(getenv)(VALGRIND_OPTS) );
       HChar* f2_clo  = NULL;
 
       // Don't read ./.valgrindrc if "." is the same as "$HOME", else its
