@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward 
+   Copyright (C) 2000-2008 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -47,6 +47,7 @@
    stage1 and stage2. */
 
 #include "pub_core_basics.h"     /* basic types */
+#include "pub_core_vkiscnums.h"  /* for syscall numbers */
 #include "pub_core_debuglog.h"   /* our own iface */
 #include "valgrind.h"            /* for RUNNING_ON_VALGRIND */
 
@@ -60,7 +61,7 @@
 
 static UInt local_sys_write_stderr ( HChar* buf, Int n )
 {
-   Int block[2];
+   volatile Int block[2];
    block[0] = (Int)buf;
    block[1] = n;
    __asm__ volatile (
@@ -69,7 +70,7 @@ static UInt local_sys_write_stderr ( HChar* buf, Int n )
       "pushl %%ebx\n"           /* save &block */
       "movl  0(%%ebx), %%ecx\n" /* %ecx = buf */
       "movl  4(%%ebx), %%edx\n" /* %edx = n */
-      "movl  $4, %%eax\n"       /* %eax = __NR_write */
+      "movl  $"VG_STRINGIFY(__NR_write)", %%eax\n" /* %eax = __NR_write */
       "movl  $1, %%ebx\n"       /* %ebx = stderr */
       "int   $0x80\n"           /* write(stderr, buf, n) */
       "popl  %%ebx\n"           /* reestablish &block */
@@ -88,7 +89,7 @@ static UInt local_sys_getpid ( void )
 {
    UInt __res;
    __asm__ volatile (
-      "movl $20, %%eax\n"  /* set %eax = __NR_getpid */
+      "movl $"VG_STRINGIFY(__NR_getpid)", %%eax\n" /* %eax = __NR_getpid */
       "int  $0x80\n"       /* getpid() */
       "movl %%eax, %0\n"   /* set __res = eax */
       : "=mr" (__res)
@@ -101,7 +102,7 @@ static UInt local_sys_getpid ( void )
 __attribute__((noinline))
 static UInt local_sys_write_stderr ( HChar* buf, Int n )
 {
-   Long block[2];
+   volatile Long block[2];
    block[0] = (Long)buf;
    block[1] = n;
    __asm__ volatile (
@@ -109,7 +110,7 @@ static UInt local_sys_write_stderr ( HChar* buf, Int n )
       "pushq %%r15\n"           /* r15 is callee-save */
       "movq  %0, %%r15\n"       /* r15 = &block */
       "pushq %%r15\n"           /* save &block */
-      "movq  $1, %%rax\n"       /* rax = __NR_write */
+      "movq  $"VG_STRINGIFY(__NR_write)", %%rax\n" /* rax = __NR_write */
       "movq  $2, %%rdi\n"       /* rdi = stderr */
       "movq  0(%%r15), %%rsi\n" /* rsi = buf */
       "movq  8(%%r15), %%rdx\n" /* rdx = n */
@@ -131,7 +132,7 @@ static UInt local_sys_getpid ( void )
 {
    UInt __res;
    __asm__ volatile (
-      "movq $39, %%rax\n"  /* set %rax = __NR_getpid */
+      "movq $"VG_STRINGIFY(__NR_getpid)", %%rax\n" /* %rax = __NR_getpid */
       "syscall\n"          /* getpid() */
       "movl %%eax, %0\n"   /* set __res = %eax */
       : "=mr" (__res)
@@ -144,14 +145,14 @@ static UInt local_sys_getpid ( void )
 
 static UInt local_sys_write_stderr ( HChar* buf, Int n )
 {
-   Int block[2];
+   volatile Int block[2];
    block[0] = (Int)buf;
    block[1] = n;
    __asm__ volatile (
       "addi 1,1,-256\n\t"
       "mr   5,%0\n\t"     /* r5 = &block[0] */
       "stw  5,0(1)\n\t"   /* stash on stack */
-      "li   0,4\n\t"      /* set %r0 = __NR_write (== 4) */
+      "li   0,"VG_STRINGIFY(__NR_write)"\n\t" /* set %r0 = __NR_write */
       "li   3,2\n\t"      /* set %r3 = stderr */
       "lwz  4,0(5)\n\t"   /* set %r4 = buf */
       "lwz  5,4(5)\n\t"   /* set %r5 = n */
@@ -176,7 +177,7 @@ static UInt local_sys_getpid ( void )
       "li 0, %1\n\t"
       "sc"
       : "=&r" (__res)
-      : "i" (20) /* == __NR_getpid */
+      : "i" (__NR_getpid)
       : "cc","memory","cr0","ctr",
         "r0","r2","r4","r5","r6","r7","r8","r9","r10","r11","r12"
    );
@@ -187,14 +188,14 @@ static UInt local_sys_getpid ( void )
 
 static UInt local_sys_write_stderr ( HChar* buf, Int n )
 {
-   Long block[2];
+   volatile Long block[2];
    block[0] = (Long)buf;
    block[1] = (Long)n;
    __asm__ volatile (
       "addi 1,1,-256\n\t"
       "mr   5,%0\n\t"     /* r5 = &block[0] */
       "std  5,0(1)\n\t"   /* stash on stack */
-      "li   0,4\n\t"      /* set %r0 = __NR_write (== 4) */
+      "li   0,"VG_STRINGIFY(__NR_write)"\n\t" /* %r0 = __NR_write */
       "li   3,2\n\t"      /* set %r3 = stderr */
       "ld   4,0(5)\n\t"   /* set %r4 = buf */
       "ld   5,8(5)\n\t"   /* set %r5 = n */
@@ -219,13 +220,179 @@ static UInt local_sys_getpid ( void )
       "li 0, %1\n\t"
       "sc"
       : "=&r" (__res)
-      : "i" (20) /* == __NR_getpid */
+      : "i" (__NR_getpid)
       : "cc","memory","cr0","ctr",
         "r0","r2","r4","r5","r6","r7","r8","r9","r10","r11","r12"
    );
    return (UInt)__res;
 }
 
+#elif defined(VGP_ppc32_aix5)
+
+static UInt local_sys_write_stderr ( HChar* buf, Int n )
+{
+   /* For some reason gcc-3.3.2 doesn't preserve r31 across the asm
+      even though we state it to be trashed.  So use r27 instead. */
+   volatile UInt block[3];
+   block[0] = (UInt)buf;
+   block[1] = n;
+   block[2] = __NR_write;
+   __asm__ __volatile__ (
+      "mr    28,%0\n\t"      /* establish base ptr */
+      "mr    27,2\n\t"       /* save r2 in r27 */
+      "mflr  30\n\t"         /* save lr in r30 */
+
+      "lwz 2,8(28)\n\t"      /* set %r2 = __NR_write */
+      "li  3,2\n\t"          /* set %r3 = stderr */
+      "lwz 4,0(28)\n\t"      /* set %r4 = buf */
+      "lwz 5,4(28)\n\t"      /* set %r5 = n */
+
+      "crorc 6,6,6\n\t"
+      ".long 0x48000005\n\t" /* bl .+4 */
+      "mflr  29\n\t"
+      "addi  29,29,16\n\t"
+      "mtlr  29\n\t"
+      "sc\n\t"               /* write() */
+
+      "stw 3,0(28)\n\t"      /* result */
+      "stw 4,4(28)\n\t"      /* error? */
+
+      "mr   2,27\n\t"        /* restore r2 */
+      "mtlr 30"              /* restore lr */
+
+      : /*out*/
+      : /*in*/  "b" (&block[0])
+      : /*trash*/
+           /*temps*/    "r31","r30","r29","r28","r27",
+           /*args*/     "r3","r4","r5","r6","r7","r8","r9","r10",
+           /*paranoia*/ "memory","cc","r0","r1","r11","r12","r13",
+                        "xer","ctr","cr0","cr1","cr2","cr3",
+                        "cr4","cr5","cr6","cr7"
+   );
+   if (block[1] != 0)
+      return -1;
+   else
+      return block[0];
+}
+
+static UInt local_sys_getpid ( void )
+{
+   /* For some reason gcc-3.3.2 doesn't preserve r31 across the asm
+      even though we state it to be trashed.  So use r27 instead. */
+   volatile UInt block[1];
+   block[0] = __NR_getpid;
+   __asm__ __volatile__ (
+      "mr    28,%0\n\t"      /* establish base ptr */
+      "mr    27,2\n\t"       /* save r2 in r27 */
+      "mflr  30\n\t"         /* save lr in r30 */
+
+      "lwz   2,0(28)\n\t"    /* set %r2 = __NR_getpid */
+
+      "crorc 6,6,6\n\t"
+      ".long 0x48000005\n\t" /* bl .+4 */
+      "mflr  29\n\t"
+      "addi  29,29,16\n\t"
+      "mtlr  29\n\t"
+      "sc\n\t"               /* getpid() */
+
+      "stw   3,0(28)\n\t"    /* result -> block[0] */
+
+      "mr   2,27\n\t"        /* restore r2 */
+      "mtlr 30"              /* restore lr */
+
+      : /*out*/
+      : /*in*/  "b" (&block[0])
+      : /*trash*/
+           /*temps*/    "r31","r30","r29","r28","r27",
+           /*args*/     "r3","r4","r5","r6","r7","r8","r9","r10",
+           /*paranoia*/ "memory","cc","r0","r1","r11","r12","r13",
+                        "xer","ctr","cr0","cr1","cr2","cr3",
+                        "cr4","cr5","cr6","cr7"
+   );
+   return block[0];
+}
+
+#elif defined(VGP_ppc64_aix5)
+
+static UInt local_sys_write_stderr ( HChar* buf, Int n )
+{
+   volatile ULong block[3];
+   block[0] = (ULong)buf;
+   block[1] = n;
+   block[2] = (ULong)__NR_write;
+   __asm__ __volatile__ (
+      "mr    28,%0\n\t"      /* establish base ptr */
+      "mr    27,2\n\t"       /* save r2 in r27 */
+      "mflr  30\n\t"         /* save lr in r30 */
+
+      "ld  2,16(28)\n\t"     /* set %r2 = __NR_write */
+      "li  3,2\n\t"          /* set %r3 = stderr */
+      "ld  4,0(28)\n\t"      /* set %r4 = buf */
+      "ld  5,8(28)\n\t"      /* set %r5 = n */
+
+      "crorc 6,6,6\n\t"
+      ".long 0x48000005\n\t" /* bl .+4 */
+      "mflr  29\n\t"
+      "addi  29,29,16\n\t"
+      "mtlr  29\n\t"
+      "sc\n\t"               /* write() */
+
+      "std 3,0(28)\n\t"      /* result */
+      "std 4,8(28)\n\t"      /* error? */
+
+      "mr   2,27\n\t"        /* restore r2 */
+      "mtlr 30"              /* restore lr */
+
+      : /*out*/
+      : /*in*/  "b" (&block[0])
+      : /*trash*/
+           /*temps*/    "r31","r30","r29","r28","r27",
+           /*args*/     "r3","r4","r5","r6","r7","r8","r9","r10",
+           /*paranoia*/ "memory","cc","r0","r1","r11","r12","r13",
+                        "xer","ctr","cr0","cr1","cr2","cr3",
+                        "cr4","cr5","cr6","cr7"
+   );
+   if (block[1] != 0)
+      return (UInt)-1;
+   else
+      return (UInt)block[0];
+}
+
+static UInt local_sys_getpid ( void )
+{
+   volatile ULong block[1];
+   block[0] = __NR_getpid;
+   __asm__ __volatile__ (
+      "mr    28,%0\n\t"      /* establish base ptr */
+      "mr    27,2\n\t"       /* save r2 in r27 */
+      "mflr  30\n\t"         /* save lr in r30 */
+
+      "ld    2,0(28)\n\t"    /* set %r2 = __NR_getpid */
+
+      "crorc 6,6,6\n\t"
+      ".long 0x48000005\n\t" /* bl .+4 */
+      "mflr  29\n\t"
+      "addi  29,29,16\n\t"
+      "mtlr  29\n\t"
+      "sc\n\t"               /* getpid() */
+
+      "std  3,0(28)\n\t"     /* result -> block[0] */
+
+      "mr   2,27\n\t"        /* restore r2 */
+      "mtlr 30"              /* restore lr */
+
+      : /*out*/
+      : /*in*/  "b" (&block[0])
+      : /*trash*/
+           /*temps*/    "r31","r30","r29","r28","r27",
+           /*args*/     "r3","r4","r5","r6","r7","r8","r9","r10",
+           /*paranoia*/ "memory","cc","r0","r1","r11","r12","r13",
+                        "xer","ctr","cr0","cr1","cr2","cr3",
+                        "cr4","cr5","cr6","cr7"
+   );
+   return (UInt)block[0];
+}
+
 #else
 # error Unknown platform
 #endif
@@ -279,7 +446,7 @@ static void emit ( HChar* buf, Int n )
 #define VG_MSG_LJUSTIFY  4 /* Must justify on the left. */
 #define VG_MSG_PAREN     8 /* Parenthesize if present (for %y) */
 #define VG_MSG_COMMA    16 /* Add commas to numbers (for %d, %u) */
-
+#define VG_MSG_ALTFORMAT 32 /* Convert the value to alternate format */
 
 /* Copy a string into the buffer. */
 static 
@@ -376,13 +543,14 @@ UInt myvprintf_int64 ( void(*send)(HChar,void*),
                        Int flags, 
                        Int base, 
                        Int width, 
+                       Bool capitalised,
                        ULong p )
 {
    HChar  buf[40];
    Int    ind = 0;
    Int    i, nc = 0;
    Bool   neg = False;
-   HChar* digits = "0123456789ABCDEF";
+   HChar* digits = capitalised ? "0123456789ABCDEF" : "0123456789abcdef";
    UInt   ret = 0;
 
    if (base < 2 || base > 16)
@@ -453,7 +621,7 @@ VG_(debugLog_vprintf) (
    Int  flags;
    Int  width;
    Int  n_ls = 0;
-   Bool is_long;
+   Bool is_long, caps;
 
    /* We assume that vargs has already been initialised by the 
       caller, using va_start, and that the caller will similarly
@@ -479,25 +647,34 @@ VG_(debugLog_vprintf) (
       flags = 0;
       n_ls  = 0;
       width = 0; /* length of the field. */
-      if (format[i] == '(') {
-         flags |= VG_MSG_PAREN;
-         i++;
-      }
-      /* If ',' follows '%', commas will be inserted. */
-      if (format[i] == ',') {
-         flags |= VG_MSG_COMMA;
-         i++;
-      }
-      /* If '-' follows '%', justify on the left. */
-      if (format[i] == '-') {
-         flags |= VG_MSG_LJUSTIFY;
-         i++;
-      }
-      /* If '0' follows '%', pads will be inserted. */
-      if (format[i] == '0') {
-         flags |= VG_MSG_ZJUSTIFY;
+      while (1) {
+         switch (format[i]) {
+         case '(':
+            flags |= VG_MSG_PAREN;
+            break;
+         case ',':
+         case '\'':
+            /* If ',' or '\'' follows '%', commas will be inserted. */
+            flags |= VG_MSG_COMMA;
+            break;
+         case '-':
+            /* If '-' follows '%', justify on the left. */
+            flags |= VG_MSG_LJUSTIFY;
+            break;
+         case '0':
+            /* If '0' follows '%', pads will be inserted. */
+            flags |= VG_MSG_ZJUSTIFY;
+            break;
+         case '#':
+            /* If '#' follows '%', alternative format will be used. */
+            flags |= VG_MSG_ALTFORMAT;
+            break;
+         default:
+            goto parse_fieldwidth;
+         }
          i++;
       }
+     parse_fieldwidth:
       /* Compute the field length. */
       while (format[i] >= '0' && format[i] <= '9') {
          width *= 10;
@@ -519,33 +696,40 @@ VG_(debugLog_vprintf) (
          case 'd': /* %d */
             flags |= VG_MSG_SIGNED;
             if (is_long)
-               ret += myvprintf_int64(send, send_arg2, flags, 10, width, 
+               ret += myvprintf_int64(send, send_arg2, flags, 10, width, False,
                                       (ULong)(va_arg (vargs, Long)));
             else
-               ret += myvprintf_int64(send, send_arg2, flags, 10, width, 
+               ret += myvprintf_int64(send, send_arg2, flags, 10, width, False,
                                       (ULong)(va_arg (vargs, Int)));
             break;
          case 'u': /* %u */
             if (is_long)
-               ret += myvprintf_int64(send, send_arg2, flags, 10, width, 
+               ret += myvprintf_int64(send, send_arg2, flags, 10, width, False,
                                       (ULong)(va_arg (vargs, ULong)));
             else
-               ret += myvprintf_int64(send, send_arg2, flags, 10, width, 
+               ret += myvprintf_int64(send, send_arg2, flags, 10, width, False,
                                       (ULong)(va_arg (vargs, UInt)));
             break;
          case 'p': /* %p */
             ret += 2;
             send('0',send_arg2);
             send('x',send_arg2);
-            ret += myvprintf_int64(send, send_arg2, flags, 16, width, 
+            ret += myvprintf_int64(send, send_arg2, flags, 16, width, True,
                                    (ULong)((UWord)va_arg (vargs, void *)));
             break;
          case 'x': /* %x */
+         case 'X': /* %X */
+            caps = toBool(format[i] == 'X');
+            if (flags & VG_MSG_ALTFORMAT) {
+               ret += 2;
+               send('0',send_arg2);
+               send('x',send_arg2);
+            }
             if (is_long)
-               ret += myvprintf_int64(send, send_arg2, flags, 16, width, 
+               ret += myvprintf_int64(send, send_arg2, flags, 16, width, caps,
                                       (ULong)(va_arg (vargs, ULong)));
             else
-               ret += myvprintf_int64(send, send_arg2, flags, 16, width, 
+               ret += myvprintf_int64(send, send_arg2, flags, 16, width, caps,
                                       (ULong)(va_arg (vargs, UInt)));
             break;
          case 'c': /* %c */
@@ -676,9 +860,9 @@ void VG_(debugLog) ( Int level, const HChar* modulename,
    }
    
    (void)myvprintf_str ( add_to_buf, &buf, 0, 2, "--", False );
-   (void)myvprintf_int64 ( add_to_buf, &buf, 0, 10, 1, (ULong)pid );
+   (void)myvprintf_int64 ( add_to_buf, &buf, 0, 10, 1, False, (ULong)pid );
    (void)myvprintf_str ( add_to_buf, &buf, 0, 1, ":", False );
-   (void)myvprintf_int64 ( add_to_buf, &buf, 0, 10, 1, (ULong)level );
+   (void)myvprintf_int64 ( add_to_buf, &buf, 0, 10, 1, False, (ULong)level );
    (void)myvprintf_str ( add_to_buf, &buf, 0, 1, ":", False );
    (void)myvprintf_str ( add_to_buf, &buf, 0, 8, (HChar*)modulename, False );
    (void)myvprintf_str ( add_to_buf, &buf, 0, indent, "", False );
