@@ -7,9 +7,9 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward 
+   Copyright (C) 2000-2008 Julian Seward 
       jseward@acm.org
-   Copyright (C) 2003-2007 Jeremy Fitzhardinge
+   Copyright (C) 2003-2008 Jeremy Fitzhardinge
       jeremy@goop.org
 
    This program is free software; you can redistribute it and/or
@@ -36,6 +36,7 @@
 #include "pub_core_libcbase.h"
 #include "pub_core_libcassert.h"
 #include "pub_core_libcprint.h"
+#include "pub_core_seqmatch.h"
 #include "pub_core_mallocfree.h"
 #include "pub_core_options.h"
 #include "pub_core_oset.h"
@@ -222,7 +223,7 @@ typedef
       HChar* from_fnpatt;  /* from fnname pattern  */
       Addr   to_addr;      /* where redirecting to */
       Bool   isWrap;       /* wrap or replacement? */
-      HChar* mandatory;    /* non-NULL ==> abort V and print the
+      const HChar* mandatory; /* non-NULL ==> abort V and print the
                               string if from_sopatt is loaded but
                               from_fnpatt cannot be found */
       /* VARIABLE PARTS -- used transiently whilst processing redirections */
@@ -239,9 +240,9 @@ typedef
 typedef
    struct _TopSpec {
       struct _TopSpec* next; /* linked list */
-      SegInfo* seginfo;      /* symbols etc */
-      Spec*    specs;        /* specs pulled out of seginfo */
-      Bool     mark; /* transient temporary used during deletion */
+      DebugInfo* seginfo;    /* symbols etc */
+      Spec*      specs;      /* specs pulled out of seginfo */
+      Bool       mark; /* transient temporary used during deletion */
    }
    TopSpec;
 
@@ -280,9 +281,9 @@ static OSet* activeSet = NULL;
 
 static void maybe_add_active ( Active /*by value; callee copies*/ );
 
-static void*  symtab_zalloc(SizeT);
-static void   symtab_free(void*);
-static HChar* symtab_strdup(HChar*);
+static void*  dinfo_zalloc(HChar* ec, SizeT);
+static void   dinfo_free(void*);
+static HChar* dinfo_strdup(HChar* ec, HChar*);
 static Bool   is_plausible_guest_addr(Addr);
 static Bool   is_aix5_glink_idiom(Addr);
 
@@ -302,19 +303,19 @@ void generate_and_add_actives (
         /* spec list and the owning TopSpec */
         Spec*    specs, 
         TopSpec* parent_spec,
-	/* seginfo and the owning TopSpec */
-        SegInfo* si,
+	/* debuginfo and the owning TopSpec */
+        DebugInfo* di,
         TopSpec* parent_sym 
      );
 
-/* Notify m_redir of the arrival of a new SegInfo.  This is fairly
+/* Notify m_redir of the arrival of a new DebugInfo.  This is fairly
    complex, but the net effect is to (1) add a new entry to the
    topspecs list, and (2) figure out what new binding are now active,
    and, as a result, add them to the actives mapping. */
 
 #define N_DEMANGLED 256
 
-void VG_(redir_notify_new_SegInfo)( SegInfo* newsi )
+void VG_(redir_notify_new_DebugInfo)( DebugInfo* newsi )
 {
    Bool         ok, isWrap;
    Int          i, nsyms;
@@ -327,6 +328,7 @@ void VG_(redir_notify_new_SegInfo)( SegInfo* newsi )
    HChar        demangled_sopatt[N_DEMANGLED];
    HChar        demangled_fnpatt[N_DEMANGLED];
    Bool         check_ppcTOCs = False;
+   Bool         isText;
    const UChar* newsi_soname;
 
 #  if defined(VG_PLAT_USES_PPCTOC)
@@ -341,7 +343,7 @@ void VG_(redir_notify_new_SegInfo)( SegInfo* newsi )
    for (ts = topSpecs; ts; ts = ts->next)
       vg_assert(ts->seginfo != newsi);
 
-   /* scan this SegInfo's symbol table, pulling out and demangling
+   /* scan this DebugInfo's symbol table, pulling out and demangling
       any specs found */
 
    specList = NULL; /* the spec list we're building up */
@@ -349,9 +351,12 @@ void VG_(redir_notify_new_SegInfo)( SegInfo* newsi )
    nsyms = VG_(seginfo_syms_howmany)( newsi );
    for (i = 0; i < nsyms; i++) {
       VG_(seginfo_syms_getidx)( newsi, i, &sym_addr, &sym_toc, 
-                                          NULL, &sym_name );
+                                          NULL, &sym_name, &isText );
       ok = VG_(maybe_Z_demangle)( sym_name, demangled_sopatt, N_DEMANGLED,
                                   demangled_fnpatt, N_DEMANGLED, &isWrap );
+      /* ignore data symbols */
+      if (!isText)
+         continue;
       if (!ok) {
          /* It's not a full-scale redirect, but perhaps it is a load-notify
             fn?  Let the load-notify department see it. */
@@ -365,10 +370,10 @@ void VG_(redir_notify_new_SegInfo)( SegInfo* newsi )
             the following loop, and complain at that point. */
          continue;
       }
-      spec = symtab_zalloc(sizeof(Spec));
+      spec = dinfo_zalloc("redir.rnnD.1", sizeof(Spec));
       vg_assert(spec);
-      spec->from_sopatt = symtab_strdup(demangled_sopatt);
-      spec->from_fnpatt = symtab_strdup(demangled_fnpatt);
+      spec->from_sopatt = dinfo_strdup("redir.rnnD.2", demangled_sopatt);
+      spec->from_fnpatt = dinfo_strdup("redir.rnnD.3", demangled_fnpatt);
       vg_assert(spec->from_sopatt);
       vg_assert(spec->from_fnpatt);
       spec->to_addr = sym_addr;
@@ -384,9 +389,11 @@ void VG_(redir_notify_new_SegInfo)( SegInfo* newsi )
    if (check_ppcTOCs) {
       for (i = 0; i < nsyms; i++) {
          VG_(seginfo_syms_getidx)( newsi, i, &sym_addr, &sym_toc, 
-                                             NULL, &sym_name );
-         ok = VG_(maybe_Z_demangle)( sym_name, demangled_sopatt, N_DEMANGLED,
-                                     demangled_fnpatt, N_DEMANGLED, &isWrap );
+                                             NULL, &sym_name, &isText );
+         ok = isText
+              && VG_(maybe_Z_demangle)( 
+                    sym_name, demangled_sopatt, N_DEMANGLED,
+                    demangled_fnpatt, N_DEMANGLED, &isWrap );
          if (!ok)
             /* not a redirect.  Ignore. */
             continue;
@@ -410,9 +417,9 @@ void VG_(redir_notify_new_SegInfo)( SegInfo* newsi )
       }
    }
 
-   /* Ok.  Now specList holds the list of specs from the SegInfo. 
+   /* Ok.  Now specList holds the list of specs from the DebugInfo. 
       Build a new TopSpec, but don't add it to topSpecs yet. */
-   newts = symtab_zalloc(sizeof(TopSpec));
+   newts = dinfo_zalloc("redir.rnnD.4", sizeof(TopSpec));
    vg_assert(newts);
    newts->next    = NULL; /* not significant */
    newts->seginfo = newsi;
@@ -458,7 +465,7 @@ void VG_(redir_notify_new_SegInfo)( SegInfo* newsi )
    topSpecs = newts;
 
    if (VG_(clo_trace_redir))
-      show_redir_state("after VG_(redir_notify_new_SegInfo)");
+      show_redir_state("after VG_(redir_notify_new_DebugInfo)");
 }
 
 #undef N_DEMANGLED
@@ -475,12 +482,12 @@ void generate_and_add_actives (
         Spec*    specs, 
         TopSpec* parent_spec,
 	/* seginfo and the owning TopSpec */
-        SegInfo* si,
+        DebugInfo* di,
         TopSpec* parent_sym 
      )
 {
    Spec*  sp;
-   Bool   anyMark;
+   Bool   anyMark, isText;
    Active act;
    Int    nsyms, i;
    Addr   sym_addr;
@@ -493,7 +500,7 @@ void generate_and_add_actives (
    for (sp = specs; sp; sp = sp->next) {
       sp->done = False;
       sp->mark = VG_(string_match)( sp->from_sopatt, 
-                                    VG_(seginfo_soname)(si) );
+                                    VG_(seginfo_soname)(di) );
       anyMark = anyMark || sp->mark;
    }
 
@@ -503,9 +510,14 @@ void generate_and_add_actives (
 
    /* Iterate outermost over the symbols in the seginfo, in the hope
       of trashing the caches less. */
-   nsyms = VG_(seginfo_syms_howmany)( si );
+   nsyms = VG_(seginfo_syms_howmany)( di );
    for (i = 0; i < nsyms; i++) {
-      VG_(seginfo_syms_getidx)( si, i, &sym_addr, NULL, NULL, &sym_name );
+      VG_(seginfo_syms_getidx)( di, i,
+                                &sym_addr, NULL, NULL, &sym_name, &isText );
+
+      /* ignore data symbols */
+      if (!isText)
+         continue;
 
       /* On AIX, we cannot redirect calls to a so-called glink
          function for reasons which are not obvious - something to do
@@ -565,7 +577,7 @@ void generate_and_add_actives (
       VG_(printf)(
       "%swas not found whilst processing\n", v);
       VG_(printf)(
-      "%ssymbols from the object with soname: %s\n", v, VG_(seginfo_soname)(si));
+      "%ssymbols from the object with soname: %s\n", v, VG_(seginfo_soname)(di));
       VG_(printf)(
       "%s\n", v);
       VG_(printf)(
@@ -632,9 +644,9 @@ static void maybe_add_active ( Active act )
          paranoia (but, I believe, unnecessarily), discard 'to' as
          well. */
       VG_(discard_translations)( (Addr64)act.from_addr, 1,
-                                 "redir_new_SegInfo(from_addr)");
+                                 "redir_new_DebugInfo(from_addr)");
       VG_(discard_translations)( (Addr64)act.to_addr, 1,
-                                 "redir_new_SegInfo(to_addr)");
+                                 "redir_new_DebugInfo(to_addr)");
    }
    return;
 
@@ -647,11 +659,11 @@ static void maybe_add_active ( Active act )
 }
 
 
-/* Notify m_redir of the deletion of a SegInfo.  This is relatively
+/* Notify m_redir of the deletion of a DebugInfo.  This is relatively
    simple -- just get rid of all actives derived from it, and free up
    the associated list elements. */
 
-void VG_(redir_notify_delete_SegInfo)( SegInfo* delsi )
+void VG_(redir_notify_delete_DebugInfo)( DebugInfo* delsi )
 {
    TopSpec* ts;
    TopSpec* tsPrev;
@@ -675,12 +687,12 @@ void VG_(redir_notify_delete_SegInfo)( SegInfo* delsi )
      ts = ts->next;
    }
 
-   vg_assert(ts); /* else we don't have the deleted SegInfo */
+   vg_assert(ts); /* else we don't have the deleted DebugInfo */
    vg_assert(ts->seginfo == delsi);
 
    /* Traverse the actives, copying the addresses of those we intend
       to delete into tmpSet. */
-   tmpSet = VG_(OSetWord_Create)(symtab_zalloc, symtab_free);
+   tmpSet = VG_(OSetWord_Create)(dinfo_zalloc, "redir.rndD.1", dinfo_free);
 
    ts->mark = True;
 
@@ -710,9 +722,9 @@ void VG_(redir_notify_delete_SegInfo)( SegInfo* delsi )
          /* While we have our hands on both the 'from' and 'to'
             of this Active, do paranoid stuff with tt/tc. */
          VG_(discard_translations)( (Addr64)act->from_addr, 1,
-                                    "redir_del_SegInfo(from_addr)");
+                                    "redir_del_DebugInfo(from_addr)");
          VG_(discard_translations)( (Addr64)act->to_addr, 1,
-                                    "redir_del_SegInfo(to_addr)");
+                                    "redir_del_DebugInfo(to_addr)");
       }
    }
 
@@ -729,10 +741,10 @@ void VG_(redir_notify_delete_SegInfo)( SegInfo* delsi )
    /* The Actives set is now cleaned up.  Free up this TopSpec and
       everything hanging off it. */
    for (sp = ts->specs; sp; sp = sp_next) {
-      if (sp->from_sopatt) symtab_free(sp->from_sopatt);
-      if (sp->from_fnpatt) symtab_free(sp->from_fnpatt);
+      if (sp->from_sopatt) dinfo_free(sp->from_sopatt);
+      if (sp->from_fnpatt) dinfo_free(sp->from_fnpatt);
       sp_next = sp->next;
-      symtab_free(sp);
+      dinfo_free(sp);
    }
 
    if (tsPrev == NULL) {
@@ -741,10 +753,10 @@ void VG_(redir_notify_delete_SegInfo)( SegInfo* delsi )
    } else {
       tsPrev->next = ts->next;
    }
-   symtab_free(ts);
+   dinfo_free(ts);
 
    if (VG_(clo_trace_redir))
-      show_redir_state("after VG_(redir_notify_delete_SegInfo)");
+      show_redir_state("after VG_(redir_notify_delete_DebugInfo)");
 }
 
 
@@ -796,13 +808,13 @@ static void add_hardwired_active ( Addr from, Addr to )
 __attribute__((unused)) /* not used on all platforms */
 static void add_hardwired_spec ( HChar* sopatt, HChar* fnpatt, 
                                  Addr   to_addr,
-                                 HChar* mandatory )
+                                 const HChar* const mandatory )
 {
-   Spec* spec = symtab_zalloc(sizeof(Spec));
+   Spec* spec = dinfo_zalloc("redir.ahs.1", sizeof(Spec));
    vg_assert(spec);
 
    if (topSpecs == NULL) {
-      topSpecs = symtab_zalloc(sizeof(TopSpec));
+      topSpecs = dinfo_zalloc("redir.ahs.2", sizeof(TopSpec));
       vg_assert(topSpecs);
       /* symtab_zalloc sets all fields to zero */
    }
@@ -828,19 +840,20 @@ static void add_hardwired_spec ( HChar* sopatt, HChar* fnpatt,
 /* Initialise the redir system, and create the initial Spec list and
    for amd64-linux a couple of permanent active mappings.  The initial
    Specs are not converted into Actives yet, on the (checked)
-   assumption that no SegInfos have so far been created, and so when
+   assumption that no DebugInfos have so far been created, and so when
    they are created, that will happen. */
 
 void VG_(redir_initialise) ( void )
 {
-   // Assert that there are no SegInfos so far
+   // Assert that there are no DebugInfos so far
    vg_assert( VG_(next_seginfo)(NULL) == NULL );
 
    // Initialise active mapping.
    activeSet = VG_(OSetGen_Create)(offsetof(Active, from_addr),
                                    NULL,     // Use fast comparison
-                                   symtab_zalloc,
-                                   symtab_free);
+                                   dinfo_zalloc,
+                                   "redir.ri.1", 
+                                   dinfo_free);
 
    // The rest of this function just adds initial Specs.   
 
@@ -867,13 +880,14 @@ void VG_(redir_initialise) ( void )
    );
 
 #  elif defined(VGP_ppc32_linux)
+   {
+   static const HChar croakage[]
+     = "Possible fix: install glibc's debuginfo package on this machine.";
+
    /* If we're using memcheck, use these intercepts right from
       the start, otherwise ld.so makes a lot of noise. */
    if (0==VG_(strcmp)("Memcheck", VG_(details).name)) {
 
-      static HChar* croakage = "Possible fix: install glibc's debuginfo "
-                               "package on this machine.";
-
       /* this is mandatory - can't sanely continue without it */
       add_hardwired_spec(
          "ld.so.1", "strlen",
@@ -892,16 +906,27 @@ void VG_(redir_initialise) ( void )
          NULL /* not mandatory - so why bother at all? */
          /* glibc-2.6.1 (openSUSE 10.3, ppc32) seems fine without it */
       );
+   } else if (0 == VG_(strcmp)("drd", VG_(details).name)) {
+      /* Only continue if symbol information in ld.so.1 is present,   */
+      /* because otherwise drd's suppression patterns on ld.so do     */
+      /* not have any effect.                                         */
+      add_hardwired_spec(
+         "ld.so.1", "strlen",
+         (Addr)(&VG_(ppc32_linux_REDIR_FOR_strlen)),
+         croakage
+      );
+   }
    }
 
 #  elif defined(VGP_ppc64_linux)
+   {
+   static const HChar croakage[]
+     = "Possible fix: install glibc's debuginfo package on this machine.";
+
    /* If we're using memcheck, use these intercepts right from
       the start, otherwise ld.so makes a lot of noise. */
    if (0==VG_(strcmp)("Memcheck", VG_(details).name)) {
 
-      static HChar* croakage = "Possible fix: install glibc's debuginfo "
-                               "package on this machine.";
-
       /* this is mandatory - can't sanely continue without it */
       add_hardwired_spec(
          "ld64.so.1", "strlen",
@@ -916,6 +941,16 @@ void VG_(redir_initialise) ( void )
          /* glibc-2.5 (FC6, ppc64) seems fine without it */
       );
 
+   } else if (0 == VG_(strcmp)("drd", VG_(details).name)) {
+      /* Only continue if symbol information in ld64.so.1 is present, */
+      /* because otherwise drd's suppression patterns on ld.so do     */
+      /* not have any effect.                                         */
+      add_hardwired_spec(
+         "ld64.so.1", "strlen",
+         (Addr)VG_(fnptr_to_fnentry)( &VG_(ppc64_linux_REDIR_FOR_strlen) ),
+         croakage
+      );
+   }
    }
 
 #  elif defined(VGP_ppc32_aix5)
@@ -937,23 +972,23 @@ void VG_(redir_initialise) ( void )
 /*--- MISC HELPERS                                         ---*/
 /*------------------------------------------------------------*/
 
-static void* symtab_zalloc(SizeT n) {
+static void* dinfo_zalloc(HChar* ec, SizeT n) {
    void* p;
    vg_assert(n > 0);
-   p = VG_(arena_malloc)(VG_AR_SYMTAB, n);
+   p = VG_(arena_malloc)(VG_AR_DINFO, ec, n);
    tl_assert(p);
    VG_(memset)(p, 0, n);
    return p;
 }
 
-static void symtab_free(void* p) {
+static void dinfo_free(void* p) {
    tl_assert(p);
-   return VG_(arena_free)(VG_AR_SYMTAB, p);
+   return VG_(arena_free)(VG_AR_DINFO, p);
 }
 
-static HChar* symtab_strdup(HChar* str)
+static HChar* dinfo_strdup(HChar* ec, HChar* str)
 {
-   return VG_(arena_strdup)(VG_AR_SYMTAB, str);
+   return VG_(arena_strdup)(VG_AR_DINFO, ec, str);
 }
 
 /* Really this should be merged with translations_allowable_from_seg
