@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2006 Julian Seward
+   Copyright (C) 2000-2008 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -31,11 +31,10 @@
 #ifndef __PRIV_SEMA_H
 #define __PRIV_SEMA_H
 
-/* Not really a semaphore, but use a pipe for a token-passing scheme */
 /* Not really a semaphore, but use a pipe for a token-passing scheme */
 typedef struct {
    Int pipe[2];
-   Int owner_thread;		/* who currently has it */
+   Int owner_lwpid;		/* who currently has it */
 } vg_sema_t;
 
 // Nb: this may be OS-specific, but let's not factor it out until we
