@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward 
+   Copyright (C) 2000-2008 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -38,6 +38,7 @@
 #include "pub_core_libcprint.h"
 #include "pub_core_machine.h"
 #include "pub_core_options.h"
+#include "pub_core_stacks.h"        // VG_(stack_limits)
 #include "pub_core_stacktrace.h"
 #include "pub_core_xarray.h"
 #include "pub_core_clientstate.h"   // VG_(client__dl_sysinfo_int80)
@@ -56,10 +57,11 @@
    first parameter, else send zero.  This helps generate better stack
    traces on ppc64-linux and has no effect on other platforms.
 */
-UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
-                            Addr* ips, UInt n_ips, 
-                            Addr ip, Addr sp, Addr fp, Addr lr,
-                            Addr fp_min, Addr fp_max_orig )
+UInt VG_(get_StackTrace_wrk) ( ThreadId tid_if_known,
+                               /*OUT*/Addr* ips, UInt n_ips,
+                               /*OUT*/Addr* sps, /*OUT*/Addr* fps,
+                               Addr ip, Addr sp, Addr fp, Addr lr,
+                               Addr fp_min, Addr fp_max_orig )
 {
 #  if defined(VGP_ppc32_linux) || defined(VGP_ppc64_linux) \
                                || defined(VGP_ppc32_aix5) \
@@ -88,20 +90,22 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
    // current page, at least.  Dunno if it helps.
    // NJN 2002-sep-17: seems to -- stack traces look like 1.0.X again
    fp_max = VG_PGROUNDUP(fp_max_orig);
-   fp_max -= sizeof(Addr);
+   if (fp_max >= sizeof(Addr))
+      fp_max -= sizeof(Addr);
 
    if (debug)
-      VG_(printf)("n_ips=%d fp_min=%p fp_max_orig=%p, fp_max=%p ip=%p fp=%p\n",
+      VG_(printf)("n_ips=%d fp_min=0x%lx fp_max_orig=0x%lx, "
+                  "fp_max=0x%lx ip=0x%lx fp=0x%lx\n",
 		  n_ips, fp_min, fp_max_orig, fp_max, ip, fp);
 
    /* Assertion broken before main() is reached in pthreaded programs;  the
     * offending stack traces only have one item.  --njn, 2002-aug-16 */
    /* vg_assert(fp_min <= fp_max);*/
-
-   if (fp_min + VG_(clo_max_stackframe) <= fp_max) {
-      /* If the stack is ridiculously big, don't poke around ... but
-         don't bomb out either.  Needed to make John Regehr's
-         user-space threads package work. JRS 20021001 */
+   if (fp_min + 512 >= fp_max) {
+      /* If the stack limits look bogus, don't poke around ... but
+         don't bomb out either. */
+      if (sps) sps[0] = sp;
+      if (fps) fps[0] = fp;
       ips[0] = ip;
       return 1;
    } 
@@ -117,6 +121,8 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
 
    /* fp is %ebp.  sp is %esp.  ip is %eip. */
 
+   if (sps) sps[0] = sp;
+   if (fps) fps[0] = fp;
    ips[0] = ip;
    i = 1;
 
@@ -150,26 +156,31 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
          fails, and is expensive. */
       /* Deal with frames resulting from functions which begin "pushl%
          ebp ; movl %esp, %ebp" which is the ABI-mandated preamble. */
-      if (fp_min <= fp && fp <= fp_max) {
+      if (fp_min <= fp && fp <= fp_max
+                                - 1 * sizeof(UWord)/*see comment below*/) {
          /* fp looks sane, so use it. */
          ip = (((UWord*)fp)[1]);
          sp = fp + sizeof(Addr) /*saved %ebp*/ 
                  + sizeof(Addr) /*ra*/;
          fp = (((UWord*)fp)[0]);
-         ips[i++] = ip;
+         if (sps) sps[i] = sp;
+         if (fps) fps[i] = fp;
+         ips[i++] = ip - 1; /* -1: refer to calling insn, not the RA */
          if (debug)
             VG_(printf)("     ipsF[%d]=0x%08lx\n", i-1, ips[i-1]);
-         ip = ip - 1;
+         ip = ip - 1; /* as per comment at the head of this loop */
          continue;
       }
 
       /* That didn't work out, so see if there is any CF info to hand
          which can be used. */
       if ( VG_(use_CF_info)( &ip, &sp, &fp, fp_min, fp_max ) ) {
-         ips[i++] = ip;
+         if (sps) sps[i] = sp;
+         if (fps) fps[i] = fp;
+         ips[i++] = ip - 1; /* -1: refer to calling insn, not the RA */
          if (debug)
             VG_(printf)("     ipsC[%d]=0x%08lx\n", i-1, ips[i-1]);
-         ip = ip - 1;
+         ip = ip - 1; /* as per comment at the head of this loop */
          continue;
       }
 
@@ -184,6 +195,8 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
    /* fp is %rbp.  sp is %rsp.  ip is %rip. */
 
    ips[0] = ip;
+   if (sps) sps[0] = sp;
+   if (fps) fps[0] = fp;
    i = 1;
 
    /* Loop unwinding the stack. Note that the IP value we get on
@@ -212,10 +225,12 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
       /* First off, see if there is any CFI info to hand which can
          be used. */
       if ( VG_(use_CF_info)( &ip, &sp, &fp, fp_min, fp_max ) ) {
-         ips[i++] = ip;
+         if (sps) sps[i] = sp;
+         if (fps) fps[i] = fp;
+         ips[i++] = ip - 1; /* -1: refer to calling insn, not the RA */
          if (debug)
-            VG_(printf)("     ipsC[%d]=%08p\n", i-1, ips[i-1]);
-         ip = ip - 1;
+            VG_(printf)("     ipsC[%d]=%#08lx\n", i-1, ips[i-1]);
+         ip = ip - 1; /* as per comment at the head of this loop */
          continue;
       }
 
@@ -227,16 +242,22 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
          the start of the fn, like GDB does, there's no reliable way
          to tell.  Hence the hack of first trying out CFI, and if that
          fails, then use this as a fallback. */
-      if (fp_min <= fp && fp <= fp_max) {
+      /* Note: re "- 1 * sizeof(UWord)", need to take account of the
+         fact that we are prodding at & ((UWord*)fp)[1] and so need to
+         adjust the limit check accordingly.  Omitting this has been
+         observed to cause segfaults on rare occasions. */
+      if (fp_min <= fp && fp <= fp_max - 1 * sizeof(UWord)) {
          /* fp looks sane, so use it. */
          ip = (((UWord*)fp)[1]);
          sp = fp + sizeof(Addr) /*saved %rbp*/ 
                  + sizeof(Addr) /*ra*/;
          fp = (((UWord*)fp)[0]);
-         ips[i++] = ip;
+         if (sps) sps[i] = sp;
+         if (fps) fps[i] = fp;
+         ips[i++] = ip - 1; /* -1: refer to calling insn, not the RA */
          if (debug)
-            VG_(printf)("     ipsF[%d]=%08p\n", i-1, ips[i-1]);
-         ip = ip - 1;
+            VG_(printf)("     ipsF[%d]=%#08lx\n", i-1, ips[i-1]);
+         ip = ip - 1; /* as per comment at the head of this loop */
          continue;
       }
 
@@ -254,10 +275,15 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
       */
       if (fp_min <= sp && sp < fp_max) {
          ip = ((UWord*)sp)[0];
-         ips[i++] = ip;
+         if (sps) sps[i] = sp;
+         if (fps) fps[i] = fp;
+         ips[i++] = ip == 0 
+                    ? 0 /* sp[0] == 0 ==> stuck at the bottom of a
+                           thread stack */
+                    : ip - 1; /* -1: refer to calling insn, not the RA */
          if (debug)
-            VG_(printf)("     ipsH[%d]=%08p\n", i-1, ips[i-1]);
-         ip = ip - 1;
+            VG_(printf)("     ipsH[%d]=%#08lx\n", i-1, ips[i-1]);
+         ip = ip - 1; /* as per comment at the head of this loop */
          sp += 8;
          continue;
       }
@@ -309,6 +335,8 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
    {
 #     define M_VG_ERRTXT 1000
       UChar buf_lr[M_VG_ERRTXT], buf_ip[M_VG_ERRTXT];
+      /* The following conditional looks grossly inefficient and
+         surely could be majorly improved, with not much effort. */
       if (VG_(get_fnname_nodemangle) (lr, buf_lr, M_VG_ERRTXT))
          if (VG_(get_fnname_nodemangle) (ip, buf_ip, M_VG_ERRTXT))
             if (VG_(strncmp)(buf_lr, buf_ip, M_VG_ERRTXT))
@@ -316,6 +344,8 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
 #     undef M_VG_ERRTXT
    }
 
+   if (sps) sps[0] = fp; /* NB. not sp */
+   if (fps) fps[0] = fp;
    ips[0] = ip;
    i = 1;
 
@@ -341,7 +371,7 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
 
          /* Try to derive a new (ip,fp) pair from the current set. */
 
-         if (fp_min <= fp && fp <= fp_max) {
+         if (fp_min <= fp && fp <= fp_max - lr_offset * sizeof(UWord)) {
             /* fp looks sane, so use it. */
 
             if (i == 1 && lr_is_first_RA)
@@ -362,7 +392,8 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
                used by the unwinding so far with 'redirs_used'. */
             if (ip == (Addr)&VG_(ppctoc_magic_redirect_return_stub)
                 && VG_(is_valid_tid)(tid_if_known)) {
-               Word hsp = VG_(threads)[tid_if_known].arch.vex.guest_REDIR_SP;
+               Word hsp = VG_(threads)[tid_if_known]
+                             .arch.vex.guest_REDIR_SP;
                hsp -= 2 * redirs_used;
                redirs_used ++;
                if (hsp >= 1 && hsp < redir_stack_size)
@@ -372,9 +403,14 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
 #           endif
 
             fp = (((UWord*)fp)[0]);
-            ips[i++] = ip;
+            if (sps) sps[i] = fp; /* NB. not sp */
+            if (fps) fps[i] = fp;
+            ips[i++] = ip - 1; /* -1: refer to calling insn, not the RA */
             if (debug)
-               VG_(printf)("     ipsF[%d]=%08p\n", i-1, ips[i-1]);
+               VG_(printf)("     ipsF[%d]=%#08lx\n", i-1, ips[i-1]);
+            ip = ip - 1; /* ip is probably dead at this point, but
+                            play safe, a la x86/amd64 above.  See
+                            extensive comments above. */
             continue;
          }
 
@@ -391,7 +427,10 @@ UInt VG_(get_StackTrace2) ( ThreadId tid_if_known,
    return n_found;
 }
 
-UInt VG_(get_StackTrace) ( ThreadId tid, StackTrace ips, UInt n_ips, 
+UInt VG_(get_StackTrace) ( ThreadId tid, 
+                           /*OUT*/StackTrace ips, UInt n_ips,
+                           /*OUT*/StackTrace sps,
+                           /*OUT*/StackTrace fps,
                            Word first_ip_delta )
 {
    /* thread in thread table */
@@ -400,6 +439,7 @@ UInt VG_(get_StackTrace) ( ThreadId tid, StackTrace ips, UInt n_ips,
    Addr sp                 = VG_(get_SP)(tid);
    Addr lr                 = VG_(get_LR)(tid);
    Addr stack_highest_word = VG_(threads)[tid].client_stack_highest_word;
+   Addr stack_lowest_word  = 0;
 
 #  if defined(VGP_x86_linux)
    /* Nasty little hack to deal with syscalls - if libc is using its
@@ -428,16 +468,22 @@ UInt VG_(get_StackTrace) ( ThreadId tid, StackTrace ips, UInt n_ips,
    }
 #  endif
 
+   /* See if we can get a better idea of the stack limits */
+   VG_(stack_limits)(sp, &stack_lowest_word, &stack_highest_word);
+
    /* Take into account the first_ip_delta. */
    vg_assert( sizeof(Addr) == sizeof(Word) );
    ip += first_ip_delta;
 
    if (0)
-      VG_(printf)("tid %d: stack_highest=0x%08lx ip=0x%08lx sp=0x%08lx fp=0x%08lx\n",
+      VG_(printf)("tid %d: stack_highest=0x%08lx ip=0x%08lx "
+                  "sp=0x%08lx fp=0x%08lx\n",
 		  tid, stack_highest_word, ip, sp, fp);
 
-   return VG_(get_StackTrace2)(tid, ips, n_ips, ip, sp, fp, lr, sp, 
-                                    stack_highest_word);
+   return VG_(get_StackTrace_wrk)(tid, ips, n_ips, 
+                                       sps, fps,
+                                       ip, sp, fp, lr, sp, 
+                                       stack_highest_word);
 }
 
 static void printIpDesc(UInt n, Addr ip)
@@ -473,8 +519,11 @@ void VG_(pp_StackTrace) ( StackTrace ips, UInt n_ips )
 void VG_(get_and_pp_StackTrace) ( ThreadId tid, UInt n_ips )
 {
    Addr ips[n_ips];
-   UInt n_ips_obtained = VG_(get_StackTrace)(tid, ips, n_ips,
-                                             0/*first_ip_delta*/);
+   UInt n_ips_obtained 
+      = VG_(get_StackTrace)(tid, ips, n_ips,
+                            NULL/*array to dump SP values in*/,
+                            NULL/*array to dump FP values in*/,
+                            0/*first_ip_delta*/);
    VG_(pp_StackTrace)(ips, n_ips_obtained);
 }
 
@@ -492,8 +541,6 @@ void VG_(apply_StackTrace)( void(*action)(UInt n, Addr ip),
    vg_assert(n_ips > 0);
    do {
       Addr ip = ips[i];
-      if (i > 0) 
-         ip -= VG_MIN_INSTR_SZB;   // point to calling line
 
       // Stop after the first appearance of "main" or one of the other names
       // (the appearance of which is a pretty good sign that we've gone past
