@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward 
+   Copyright (C) 2000-2008 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -30,19 +30,23 @@
 
 #include "libvex_guest_offsets.h"
 #include "pub_core_basics.h"
+#include "pub_core_aspacemgr.h"
+#include "pub_core_vki.h"
+#include "pub_core_vkiscnums.h"
 #include "pub_core_threadstate.h"
 #include "pub_core_libcbase.h"
 #include "pub_core_libcassert.h"
 #include "pub_core_libcprint.h"
 #include "pub_core_libcproc.h"      // For VG_(getpid)()
 #include "pub_core_libcsignal.h"
-#include "pub_core_scheduler.h"     // For VG_(set_sleeping), VG_(set_running),
+#include "pub_core_scheduler.h"     // For VG_({acquire,release}_BigLock),
                                     //   and VG_(vg_yield)
 #include "pub_core_stacktrace.h"    // For VG_(get_and_pp_StackTrace)()
 #include "pub_core_tooliface.h"
 #include "pub_core_options.h"
 #include "pub_core_signals.h"       // For VG_SIGVGKILL, VG_(poll_signals)
 #include "pub_core_syscall.h"
+#include "pub_core_machine.h"
 #include "pub_core_syswrap.h"
 
 #include "priv_types_n_macros.h"
@@ -50,13 +54,17 @@
 
 
 /* Useful info which needs to be recorded somewhere:
-   Use of registers in syscalls (on linux) is:
+   Use of registers in syscalls is:
 
-          NUM  ARG1  ARG2  ARG3  ARG4  ARG5  ARG6  RESULT
-   x86    eax  ebx   ecx   edx   esi   edi   ebp   eax       (== NUM)
-   amd64  rax  rdi   rsi   rdx   r10   r8    r9    rax       (== NUM)
-   ppc32  r0   r3    r4    r5    r6    r7    r8    r3+CR0.SO (== ARG1)
-   ppc64  r0   r3    r4    r5    r6    r7    r8    r3+CR0.SO (== ARG1)
+          NUM ARG1 ARG2 ARG3 ARG4 ARG5 ARG6 ARG7 ARG8 RESULT
+   LINUX:
+   x86    eax ebx  ecx  edx  esi  edi  ebp  n/a  n/a  eax       (== NUM)
+   amd64  rax rdi  rsi  rdx  r10  r8   r9   n/a  n/a  rax       (== NUM)
+   ppc32  r0  r3   r4   r5   r6   r7   r8   n/a  n/a  r3+CR0.SO (== ARG1)
+   ppc64  r0  r3   r4   r5   r6   r7   r8   n/a  n/a  r3+CR0.SO (== ARG1)
+   AIX:
+   ppc32  r2  r3   r4   r5   r6   r7   r8   r9   r10  r3(res),r4(err)
+   ppc64  r2  r3   r4   r5   r6   r7   r8   r9   r10  r3(res),r4(err)
 */
 
 /* This is the top level of the system-call handler module.  All
@@ -76,6 +84,9 @@
    whenever a client thread wants to do a syscall.  The following is a
    sketch of what it does.
 
+   * Ensures the root thread's stack is suitably mapped.  Tedious and
+     arcane.  See big big comment in VG_(client_syscall).
+
    * First, it rounds up the syscall number and args (which is a
      platform dependent activity) and puts them in a struct ("args")
      and also a copy in "orig_args".
@@ -223,11 +234,15 @@
    VG_(fixup_guest_state_after_syscall_interrupted) below for details.
 */
 extern
-UWord ML_(do_syscall_for_client_WRK)( Int syscallno, 
+UWord ML_(do_syscall_for_client_WRK)( Word syscallno, 
                                       void* guest_state,
                                       const vki_sigset_t *syscall_mask,
                                       const vki_sigset_t *restore_mask,
-                                      Int nsigwords );
+                                      Word nsigwords
+#                                     if defined(VGO_aix5)
+                                      , Word __nr_sigprocmask
+#                                     endif
+                                    );
 
 static
 void do_syscall_for_client ( Int syscallno,
@@ -239,6 +254,9 @@ void do_syscall_for_client ( Int syscallno,
       = ML_(do_syscall_for_client_WRK)(
            syscallno, &tst->arch.vex, 
            syscall_mask, &saved, _VKI_NSIG_WORDS * sizeof(UWord)
+#          if defined(VGO_aix5)
+           , __NR_rt_sigprocmask
+#          endif
         );
    vg_assert2(
       err == 0,
@@ -262,38 +280,28 @@ Bool eq_SyscallArgs ( SyscallArgs* a1, SyscallArgs* a2 )
           && a1->arg3 == a2->arg3
           && a1->arg4 == a2->arg4
           && a1->arg5 == a2->arg5
-          && a1->arg6 == a2->arg6;
+          && a1->arg6 == a2->arg6
+          && a1->arg7 == a2->arg7
+          && a1->arg8 == a2->arg8;
 }
 
 static
 Bool eq_SyscallStatus ( SyscallStatus* s1, SyscallStatus* s2 )
 {
    return s1->what == s2->what 
-          && s1->val == s2->val;
+          && s1->sres.res == s2->sres.res
+          && s1->sres.err == s2->sres.err;
 }
 
 
-/* Convert between SysRet and SyscallStatus, to the extent possible. */
-
-/* This is unused. */
-/*
-static
-SysRes convert_SyscallStatus_to_SysRes ( SyscallStatus status )
-{
-   SysRes res;
-   vg_assert(status.what == SsSuccess || status.what == SsFailure);
-   res.isError = status.what == SsFailure;
-   res.val     = status.val;
-   return res;
-}
-*/
+/* Convert between SysRes and SyscallStatus, to the extent possible. */
 
 static
 SyscallStatus convert_SysRes_to_SyscallStatus ( SysRes res )
 {
    SyscallStatus status;
-   status.what = res.isError ? SsFailure : SsSuccess;
-   status.val  = res.val;
+   status.what = SsComplete;
+   status.sres = res;
    return status;
 }
 
@@ -315,6 +323,8 @@ void getSyscallArgsFromGuestState ( /*OUT*/SyscallArgs*       canonical,
    canonical->arg4  = gst->guest_ESI;
    canonical->arg5  = gst->guest_EDI;
    canonical->arg6  = gst->guest_EBP;
+   canonical->arg7  = 0;
+   canonical->arg8  = 0;
 
 #elif defined(VGP_amd64_linux)
    VexGuestAMD64State* gst = (VexGuestAMD64State*)gst_vanilla;
@@ -325,6 +335,9 @@ void getSyscallArgsFromGuestState ( /*OUT*/SyscallArgs*       canonical,
    canonical->arg4  = gst->guest_R10;
    canonical->arg5  = gst->guest_R8;
    canonical->arg6  = gst->guest_R9;
+   canonical->arg7  = 0;
+   canonical->arg8  = 0;
+
 
 #elif defined(VGP_ppc32_linux)
    VexGuestPPC32State* gst = (VexGuestPPC32State*)gst_vanilla;
@@ -335,6 +348,9 @@ void getSyscallArgsFromGuestState ( /*OUT*/SyscallArgs*       canonical,
    canonical->arg4  = gst->guest_GPR6;
    canonical->arg5  = gst->guest_GPR7;
    canonical->arg6  = gst->guest_GPR8;
+   canonical->arg7  = 0;
+   canonical->arg8  = 0;
+
 
 #elif defined(VGP_ppc64_linux)
    VexGuestPPC64State* gst = (VexGuestPPC64State*)gst_vanilla;
@@ -345,6 +361,33 @@ void getSyscallArgsFromGuestState ( /*OUT*/SyscallArgs*       canonical,
    canonical->arg4  = gst->guest_GPR6;
    canonical->arg5  = gst->guest_GPR7;
    canonical->arg6  = gst->guest_GPR8;
+   canonical->arg7  = 0;
+   canonical->arg8  = 0;
+
+
+#elif defined(VGP_ppc32_aix5)
+   VexGuestPPC32State* gst = (VexGuestPPC32State*)gst_vanilla;
+   canonical->sysno = gst->guest_GPR2;
+   canonical->arg1  = gst->guest_GPR3;
+   canonical->arg2  = gst->guest_GPR4;
+   canonical->arg3  = gst->guest_GPR5;
+   canonical->arg4  = gst->guest_GPR6;
+   canonical->arg5  = gst->guest_GPR7;
+   canonical->arg6  = gst->guest_GPR8;
+   canonical->arg7  = gst->guest_GPR9;
+   canonical->arg8  = gst->guest_GPR10;
+
+#elif defined(VGP_ppc64_aix5)
+   VexGuestPPC64State* gst = (VexGuestPPC64State*)gst_vanilla;
+   canonical->sysno = gst->guest_GPR2;
+   canonical->arg1  = gst->guest_GPR3;
+   canonical->arg2  = gst->guest_GPR4;
+   canonical->arg3  = gst->guest_GPR5;
+   canonical->arg4  = gst->guest_GPR6;
+   canonical->arg5  = gst->guest_GPR7;
+   canonical->arg6  = gst->guest_GPR8;
+   canonical->arg7  = gst->guest_GPR9;
+   canonical->arg8  = gst->guest_GPR10;
 
 #else
 #  error "getSyscallArgsFromGuestState: unknown arch"
@@ -395,6 +438,30 @@ void putSyscallArgsIntoGuestState ( /*IN*/ SyscallArgs*       canonical,
    gst->guest_GPR7 = canonical->arg5;
    gst->guest_GPR8 = canonical->arg6;
 
+#elif defined(VGP_ppc32_aix5)
+   VexGuestPPC32State* gst = (VexGuestPPC32State*)gst_vanilla;
+   gst->guest_GPR2  = canonical->sysno;
+   gst->guest_GPR3  = canonical->arg1;
+   gst->guest_GPR4  = canonical->arg2;
+   gst->guest_GPR5  = canonical->arg3;
+   gst->guest_GPR6  = canonical->arg4;
+   gst->guest_GPR7  = canonical->arg5;
+   gst->guest_GPR8  = canonical->arg6;
+   gst->guest_GPR9  = canonical->arg7;
+   gst->guest_GPR10 = canonical->arg8;
+
+#elif defined(VGP_ppc64_aix5)
+   VexGuestPPC64State* gst = (VexGuestPPC64State*)gst_vanilla;
+   gst->guest_GPR2  = canonical->sysno;
+   gst->guest_GPR3  = canonical->arg1;
+   gst->guest_GPR4  = canonical->arg2;
+   gst->guest_GPR5  = canonical->arg3;
+   gst->guest_GPR6  = canonical->arg4;
+   gst->guest_GPR7  = canonical->arg5;
+   gst->guest_GPR8  = canonical->arg6;
+   gst->guest_GPR9  = canonical->arg7;
+   gst->guest_GPR10 = canonical->arg8;
+
 #else
 #  error "putSyscallArgsIntoGuestState: unknown arch"
 #endif
@@ -404,103 +471,118 @@ static
 void getSyscallStatusFromGuestState ( /*OUT*/SyscallStatus*     canonical,
                                       /*IN*/ VexGuestArchState* gst_vanilla )
 {
-#if defined(VGP_x86_linux)
+#  if defined(VGP_x86_linux)
    VexGuestX86State* gst = (VexGuestX86State*)gst_vanilla;
-   Int               i   = (Int)gst->guest_EAX;
-   canonical->what = i >= -4095 && i <= -1  ? SsFailure  : SsSuccess;
-   canonical->val  = (UWord)(canonical->what==SsFailure ? -i : i);
+   canonical->sres = VG_(mk_SysRes_x86_linux)( gst->guest_EAX );
+   canonical->what = SsComplete;
 
-#elif defined(VGP_amd64_linux)
+#  elif defined(VGP_amd64_linux)
    VexGuestAMD64State* gst = (VexGuestAMD64State*)gst_vanilla;
-   Long                i   = (Long)gst->guest_RAX;
-   canonical->what = i >= -4095 && i <= -1  ? SsFailure  : SsSuccess;
-   canonical->val  = (UWord)(canonical->what==SsFailure ? -i : i);
+   canonical->sres = VG_(mk_SysRes_amd64_linux)( gst->guest_RAX );
+   canonical->what = SsComplete;
 
-#elif defined(VGP_ppc32_linux)
+#  elif defined(VGP_ppc32_linux)
+   VexGuestPPC32State* gst   = (VexGuestPPC32State*)gst_vanilla;
+   UInt                cr    = LibVEX_GuestPPC32_get_CR( gst );
+   UInt                cr0so = (cr >> 28) & 1;
+   canonical->sres = VG_(mk_SysRes_ppc32_linux)( gst->guest_GPR3, cr0so );
+   canonical->what = SsComplete;
+
+#  elif defined(VGP_ppc64_linux)
+   VexGuestPPC64State* gst   = (VexGuestPPC64State*)gst_vanilla;
+   UInt                cr    = LibVEX_GuestPPC64_get_CR( gst );
+   UInt                cr0so = (cr >> 28) & 1;
+   canonical->sres = VG_(mk_SysRes_ppc64_linux)( gst->guest_GPR3, cr0so );
+   canonical->what = SsComplete;
+
+#  elif defined(VGP_ppc32_aix5)
    VexGuestPPC32State* gst = (VexGuestPPC32State*)gst_vanilla;
-   UInt                cr  = LibVEX_GuestPPC32_get_CR( gst );
-   UInt                err = (cr >> 28) & 1;  // CR0.SO
-   canonical->what = (err == 1)  ? SsFailure  : SsSuccess;
-   canonical->val  = (UWord)gst->guest_GPR3;
+   canonical->sres = VG_(mk_SysRes_ppc32_aix5)( gst->guest_GPR3, 
+                                                gst->guest_GPR4 );
+   canonical->what = SsComplete;
 
-#elif defined(VGP_ppc64_linux)
+#  elif defined(VGP_ppc64_aix5)
    VexGuestPPC64State* gst = (VexGuestPPC64State*)gst_vanilla;
-   UInt                cr  = LibVEX_GuestPPC64_get_CR( gst );
-   UInt                err = (cr >> 28) & 1;  // CR0.SO
-   canonical->what = (err == 1)  ? SsFailure  : SsSuccess;
-   canonical->val  = (UWord)gst->guest_GPR3;
+   canonical->sres = VG_(mk_SysRes_ppc64_aix5)( gst->guest_GPR3, 
+                                                gst->guest_GPR4 );
+   canonical->what = SsComplete;
 
-#else
-#  error "getSyscallStatusFromGuestState: unknown arch"
-#endif
+#  else
+#    error "getSyscallStatusFromGuestState: unknown arch"
+#  endif
 }
 
 static 
 void putSyscallStatusIntoGuestState ( /*IN*/ SyscallStatus*     canonical,
                                       /*OUT*/VexGuestArchState* gst_vanilla )
 {
-#if defined(VGP_x86_linux)
+#  if defined(VGP_x86_linux)
    VexGuestX86State* gst = (VexGuestX86State*)gst_vanilla;
-   vg_assert(canonical->what == SsSuccess 
-             || canonical->what == SsFailure);
-   if (canonical->what == SsFailure) {
+   vg_assert(canonical->what == SsComplete);
+   if (canonical->sres.isError) {
       /* This isn't exactly right, in that really a Failure with res
          not in the range 1 .. 4095 is unrepresentable in the
          Linux-x86 scheme.  Oh well. */
-      gst->guest_EAX = - (Int)canonical->val;
+      gst->guest_EAX = - (Int)canonical->sres.err;
    } else {
-      gst->guest_EAX = canonical->val;
-   }
-#elif defined(VGP_amd64_linux)
-   VexGuestAMD64State* gst = (VexGuestAMD64State*)gst_vanilla;
-   vg_assert(canonical->what == SsSuccess 
-             || canonical->what == SsFailure);
-   if (canonical->what == SsFailure) {
-      /* This isn't exactly right, in that really a Failure with res
-         not in the range 1 .. 4095 is unrepresentable in the
-         Linux-x86 scheme.  Oh well. */
-      gst->guest_RAX = - (Long)canonical->val;
-   } else {
-      gst->guest_RAX = canonical->val;
+      gst->guest_EAX = canonical->sres.res;
    }
 
-#elif defined(VGP_ppc32_linux)
+#  elif defined(VGP_amd64_linux)
+   VexGuestAMD64State* gst = (VexGuestAMD64State*)gst_vanilla;
+   vg_assert(canonical->what == SsComplete);
+   if (canonical->sres.isError) {
+      /* This isn't exactly right, in that really a Failure with res
+         not in the range 1 .. 4095 is unrepresentable in the
+         Linux-x86 scheme.  Oh well. */
+      gst->guest_RAX = - (Long)canonical->sres.err;
+   } else {
+      gst->guest_RAX = canonical->sres.res;
+   }
+
+#  elif defined(VGP_ppc32_linux)
    VexGuestPPC32State* gst = (VexGuestPPC32State*)gst_vanilla;
    UInt old_cr = LibVEX_GuestPPC32_get_CR(gst);
-
-   vg_assert(canonical->what == SsSuccess 
-             || canonical->what == SsFailure);
-
-   gst->guest_GPR3 = canonical->val;
-
-   if (canonical->what == SsFailure) {
+   vg_assert(canonical->what == SsComplete);
+   if (canonical->sres.isError) {
       /* set CR0.SO */
       LibVEX_GuestPPC32_put_CR( old_cr | (1<<28), gst );
+      gst->guest_GPR3 = canonical->sres.err;
    } else {
       /* clear CR0.SO */
       LibVEX_GuestPPC32_put_CR( old_cr & ~(1<<28), gst );
+      gst->guest_GPR3 = canonical->sres.res;
    }
 
-#elif defined(VGP_ppc64_linux)
+#  elif defined(VGP_ppc64_linux)
    VexGuestPPC64State* gst = (VexGuestPPC64State*)gst_vanilla;
    UInt old_cr = LibVEX_GuestPPC64_get_CR(gst);
-
-   vg_assert(canonical->what == SsSuccess 
-             || canonical->what == SsFailure);
-
-   gst->guest_GPR3 = canonical->val;
-
-   if (canonical->what == SsFailure) {
+   vg_assert(canonical->what == SsComplete);
+   if (canonical->sres.isError) {
       /* set CR0.SO */
       LibVEX_GuestPPC64_put_CR( old_cr | (1<<28), gst );
+      gst->guest_GPR3 = canonical->sres.err;
    } else {
       /* clear CR0.SO */
       LibVEX_GuestPPC64_put_CR( old_cr & ~(1<<28), gst );
+      gst->guest_GPR3 = canonical->sres.res;
    }
 
-#else
-#  error "putSyscallStatusIntoGuestState: unknown arch"
-#endif
+#  elif defined(VGP_ppc32_aix5)
+   VexGuestPPC32State* gst = (VexGuestPPC32State*)gst_vanilla;
+   vg_assert(canonical->what == SsComplete);
+   gst->guest_GPR3 = canonical->sres.res;
+   gst->guest_GPR4 = canonical->sres.err;
+
+#  elif defined(VGP_ppc64_aix5)
+   VexGuestPPC64State* gst = (VexGuestPPC64State*)gst_vanilla;
+   vg_assert(canonical->what == SsComplete);
+   gst->guest_GPR3 = canonical->sres.res;
+   gst->guest_GPR4 = canonical->sres.err;
+
+#  else
+#    error "putSyscallStatusIntoGuestState: unknown arch"
+#  endif
 }
 
 
@@ -519,6 +601,8 @@ void getSyscallArgLayout ( /*OUT*/SyscallArgLayout* layout )
    layout->o_arg4   = OFFSET_x86_ESI;
    layout->o_arg5   = OFFSET_x86_EDI;
    layout->o_arg6   = OFFSET_x86_EBP;
+   layout->o_arg7   = -1; /* impossible value */
+   layout->o_arg8   = -1; /* impossible value */
    layout->o_retval = OFFSET_x86_EAX;
 
 #elif defined(VGP_amd64_linux)
@@ -529,6 +613,8 @@ void getSyscallArgLayout ( /*OUT*/SyscallArgLayout* layout )
    layout->o_arg4   = OFFSET_amd64_R10;
    layout->o_arg5   = OFFSET_amd64_R8;
    layout->o_arg6   = OFFSET_amd64_R9;
+   layout->o_arg7   = -1; /* impossible value */
+   layout->o_arg8   = -1; /* impossible value */
    layout->o_retval = OFFSET_amd64_RAX;
 
 #elif defined(VGP_ppc32_linux)
@@ -539,6 +625,8 @@ void getSyscallArgLayout ( /*OUT*/SyscallArgLayout* layout )
    layout->o_arg4   = OFFSET_ppc32_GPR6;
    layout->o_arg5   = OFFSET_ppc32_GPR7;
    layout->o_arg6   = OFFSET_ppc32_GPR8;
+   layout->o_arg7   = -1; /* impossible value */
+   layout->o_arg8   = -1; /* impossible value */
    layout->o_retval = OFFSET_ppc32_GPR3;
 
 #elif defined(VGP_ppc64_linux)
@@ -549,6 +637,32 @@ void getSyscallArgLayout ( /*OUT*/SyscallArgLayout* layout )
    layout->o_arg4   = OFFSET_ppc64_GPR6;
    layout->o_arg5   = OFFSET_ppc64_GPR7;
    layout->o_arg6   = OFFSET_ppc64_GPR8;
+   layout->o_arg7   = -1; /* impossible value */
+   layout->o_arg8   = -1; /* impossible value */
+   layout->o_retval = OFFSET_ppc64_GPR3;
+
+#elif defined(VGP_ppc32_aix5)
+   layout->o_sysno  = OFFSET_ppc32_GPR2;
+   layout->o_arg1   = OFFSET_ppc32_GPR3;
+   layout->o_arg2   = OFFSET_ppc32_GPR4;
+   layout->o_arg3   = OFFSET_ppc32_GPR5;
+   layout->o_arg4   = OFFSET_ppc32_GPR6;
+   layout->o_arg5   = OFFSET_ppc32_GPR7;
+   layout->o_arg6   = OFFSET_ppc32_GPR8;
+   layout->o_arg7   = OFFSET_ppc32_GPR9;
+   layout->o_arg8   = OFFSET_ppc32_GPR10;
+   layout->o_retval = OFFSET_ppc32_GPR3;
+
+#elif defined(VGP_ppc64_aix5)
+   layout->o_sysno  = OFFSET_ppc64_GPR2;
+   layout->o_arg1   = OFFSET_ppc64_GPR3;
+   layout->o_arg2   = OFFSET_ppc64_GPR4;
+   layout->o_arg3   = OFFSET_ppc64_GPR5;
+   layout->o_arg4   = OFFSET_ppc64_GPR6;
+   layout->o_arg5   = OFFSET_ppc64_GPR7;
+   layout->o_arg6   = OFFSET_ppc64_GPR8;
+   layout->o_arg7   = OFFSET_ppc64_GPR9;
+   layout->o_arg8   = OFFSET_ppc64_GPR10;
    layout->o_retval = OFFSET_ppc64_GPR3;
 
 #else
@@ -573,6 +687,11 @@ void bad_before ( ThreadId              tid,
 {
    VG_(message)
       (Vg_DebugMsg,"WARNING: unhandled syscall: %llu", (ULong)args->sysno);
+#  if defined(VGO_aix5)
+   VG_(message)
+      (Vg_DebugMsg,"           name of syscall: \"%s\"",
+                    VG_(aix5_sysno_to_sysname)(args->sysno));
+#  endif
    if (VG_(clo_verbosity) > 1) {
       VG_(get_and_pp_StackTrace)(tid, VG_(clo_backtrace_size));
    }
@@ -580,6 +699,10 @@ void bad_before ( ThreadId              tid,
       (Vg_DebugMsg,"You may be able to write your own handler.");
    VG_(message)
       (Vg_DebugMsg,"Read the file README_MISSING_SYSCALL_OR_IOCTL.");
+   VG_(message)
+      (Vg_DebugMsg,"Nevertheless we consider this a bug.  Please report");
+   VG_(message)
+      (Vg_DebugMsg,"it at http://valgrind.org/support/bug_reports.html.");
 
    SET_STATUS_Failure(VKI_ENOSYS);
 }
@@ -589,13 +712,24 @@ static SyscallTableEntry bad_sys =
 
 static const SyscallTableEntry* get_syscall_entry ( UInt syscallno )
 {
-   const SyscallTableEntry* sys = &bad_sys;
+   const SyscallTableEntry* sys = NULL;
 
+#  if defined(VGO_linux)
    if (syscallno < ML_(syscall_table_size) &&
        ML_(syscall_table)[syscallno].before != NULL)
       sys = &ML_(syscall_table)[syscallno];
 
-   return sys;
+#  elif defined(VGP_ppc32_aix5)
+   sys = ML_(get_ppc32_aix5_syscall_entry) ( syscallno );
+
+#  elif defined(VGP_ppc64_aix5)
+   sys = ML_(get_ppc64_aix5_syscall_entry) ( syscallno );
+
+#  else
+#    error Unknown OS
+#  endif
+
+   return sys == NULL  ? &bad_sys  : sys;
 }
 
 
@@ -660,6 +794,93 @@ void VG_(client_syscall) ( ThreadId tid )
 
    tst = VG_(get_ThreadState)(tid);
 
+   /* BEGIN ensure root thread's stack is suitably mapped */
+   /* In some rare circumstances, we may do the syscall without the
+      bottom page of the stack being mapped, because the stack pointer
+      was moved down just a few instructions before the syscall
+      instruction, and there have been no memory references since
+      then, that would cause a call to VG_(extend_stack) to have
+      happened.
+
+      In native execution that's OK: the kernel automagically extends
+      the stack's mapped area down to cover the stack pointer (or sp -
+      redzone, really).  In simulated normal execution that's OK too,
+      since any signals we get from accessing below the mapped area of
+      the (guest's) stack lead us to VG_(extend_stack), where we
+      simulate the kernel's stack extension logic.  But that leaves
+      the problem of entering a syscall with the SP unmapped.  Because
+      the kernel doesn't know that the segment immediately above SP is
+      supposed to be a grow-down segment, it causes the syscall to
+      fail, and thereby causes a divergence between native behaviour
+      (syscall succeeds) and simulated behaviour (syscall fails).
+
+      This is quite a rare failure mode.  It has only been seen
+      affecting calls to sys_readlink on amd64-linux, and even then it
+      requires a certain code sequence around the syscall to trigger
+      it.  Here is one:
+
+      extern int my_readlink ( const char* path );
+      asm(
+      ".text\n"
+      ".globl my_readlink\n"
+      "my_readlink:\n"
+      "\tsubq    $0x1008,%rsp\n"
+      "\tmovq    %rdi,%rdi\n"              // path is in rdi
+      "\tmovq    %rsp,%rsi\n"              // &buf[0] -> rsi
+      "\tmovl    $0x1000,%edx\n"           // sizeof(buf) in rdx
+      "\tmovl    $"__NR_READLINK",%eax\n"  // syscall number
+      "\tsyscall\n"
+      "\taddq    $0x1008,%rsp\n"
+      "\tret\n"
+      ".previous\n"
+      );
+
+      For more details, see bug #156404
+      (https://bugs.kde.org/show_bug.cgi?id=156404).
+
+      The fix is actually very simple.  We simply need to call
+      VG_(extend_stack) for this thread, handing it the lowest
+      possible valid address for stack (sp - redzone), to ensure the
+      pages all the way down to that address, are mapped.  Because
+      this is a potentially expensive and frequent operation, we
+      filter in two ways:
+
+      First, only the main thread (tid=1) has a growdown stack.  So
+      ignore all others.  It is conceivable, although highly unlikely,
+      that the main thread exits, and later another thread is
+      allocated tid=1, but that's harmless, I believe;
+      VG_(extend_stack) will do nothing when applied to a non-root
+      thread.
+
+      Secondly, first call VG_(am_find_nsegment) directly, to see if
+      the page holding (sp - redzone) is mapped correctly.  If so, do
+      nothing.  This is almost always the case.  VG_(extend_stack)
+      calls VG_(am_find_nsegment) twice, so this optimisation -- and
+      that's all it is -- more or less halves the number of calls to
+      VG_(am_find_nsegment) required.
+
+      TODO: the test "seg->kind == SkAnonC" is really inadequate,
+      because although it tests whether the segment is mapped
+      _somehow_, it doesn't check that it has the right permissions
+      (r,w, maybe x) ?  We could test that here, but it will also be
+      necessary to fix the corresponding test in VG_(extend_stack).
+
+      All this guff is of course Linux-specific.  Hence the ifdef.
+   */
+#  if defined(VGO_linux)
+   if (tid == 1/*ROOT THREAD*/) {
+      Addr     stackMin   = VG_(get_SP)(tid) - VG_STACK_REDZONE_SZB;
+      NSegment const* seg = VG_(am_find_nsegment)(stackMin);
+      if (seg && seg->kind == SkAnonC) {
+         /* stackMin is already mapped.  Nothing to do. */
+      } else {
+         (void)VG_(extend_stack)( stackMin,
+                                  tst->client_stack_szB );
+      }
+   }
+#  endif
+   /* END ensure root thread's stack is suitably mapped */
+
    /* First off, get the syscall args and number.  This is a
       platform-dependent action. */
 
@@ -677,9 +898,11 @@ void VG_(client_syscall) ( ThreadId tid )
    sysno = sci->orig_args.sysno;
 
    /* The default what-to-do-next thing is hand the syscall to the
-      kernel, so we pre-set that here. */
+      kernel, so we pre-set that here.  Set .sres to something
+      harmless looking (is irrelevant because .what is not
+      SsComplete.) */
    sci->status.what = SsHandToKernel;
-   sci->status.val  = 0;
+   sci->status.sres = VG_(mk_SysRes_Error)(0);
    sci->flags       = 0;
 
    /* Fetch the syscall's handlers.  If no handlers exist for this
@@ -730,26 +953,30 @@ void VG_(client_syscall) ( ThreadId tid )
    */
    /* Now we proceed according to what the pre-handler decided. */
    vg_assert(sci->status.what == SsHandToKernel
-             || sci->status.what == SsSuccess
-             || sci->status.what == SsFailure);
+             || sci->status.what == SsComplete);
    vg_assert(sci->args.sysno == sci->orig_args.sysno);
 
-   if (sci->status.what == SsSuccess) {
+   if (sci->status.what == SsComplete && !sci->status.sres.isError) {
       /* The pre-handler completed the syscall itself, declaring
          success. */
-      PRINT(" --> [pre-success] Success(0x%llx)\n", (Long)sci->status.val );
-                                       
+      if (sci->flags & SfNoWriteResult) {
+         PRINT(" --> [pre-success] NoWriteResult\n");
+      } else {
+         PRINT(" --> [pre-success] Success(0x%llx)\n",
+               (ULong)sci->status.sres.res );
+      }                                      
       /* In this case the allowable flags are to ask for a signal-poll
          and/or a yield after the call.  Changing the args isn't
          allowed. */
-      vg_assert(0 == (sci->flags & ~(SfPollAfter | SfYieldAfter)));
+      vg_assert(0 == (sci->flags 
+                      & ~(SfPollAfter | SfYieldAfter | SfNoWriteResult)));
       vg_assert(eq_SyscallArgs(&sci->args, &sci->orig_args));
    }
 
    else
-   if (sci->status.what == SsFailure) {
+   if (sci->status.what == SsComplete && sci->status.sres.isError) {
       /* The pre-handler decided to fail syscall itself. */
-      PRINT(" --> [pre-fail] Failure(0x%llx)\n", (Long)sci->status.val );
+      PRINT(" --> [pre-fail] Failure(0x%llx)\n", (ULong)sci->status.sres.err );
       /* In this case, the pre-handler is also allowed to ask for the
          post-handler to be run anyway.  Changing the args is not
          allowed. */
@@ -794,7 +1021,7 @@ void VG_(client_syscall) ( ThreadId tid )
          putSyscallArgsIntoGuestState( &sci->args, &tst->arch.vex );
 
          /* Drop the lock */
-         VG_(set_sleeping)(tid, VgTs_WaitSys);
+         VG_(release_BigLock)(tid, VgTs_WaitSys, "VG_(client_syscall)[async]");
 
          /* Do the call, which operates directly on the guest state,
             not on our abstracted copies of the args/result. */
@@ -809,16 +1036,18 @@ void VG_(client_syscall) ( ThreadId tid )
             to the scheduler.  */
 
          /* Reacquire the lock */
-         VG_(set_running)(tid);
+         VG_(acquire_BigLock)(tid, "VG_(client_syscall)[async]");
 
          /* Even more impedance matching.  Extract the syscall status
             from the guest state. */
          getSyscallStatusFromGuestState( &sci->status, &tst->arch.vex );
+         vg_assert(sci->status.what == SsComplete);
 
-         PRINT("SYSCALL[%d,%d](%3d) ... [async] --> %s(0x%llx)\n",
+         PRINT("SYSCALL[%d,%d](%3ld) ... [async] --> %s(0x%llx)\n",
                VG_(getpid)(), tid, sysno, 
-               sci->status.what==SsSuccess ? "Success" : "Failure",
-               (Long)sci->status.val );
+               sci->status.sres.isError ? "Failure" : "Success",
+               sci->status.sres.isError ? (ULong)sci->status.sres.err
+                                        : (ULong)sci->status.sres.res );
 
       } else {
 
@@ -828,27 +1057,28 @@ void VG_(client_syscall) ( ThreadId tid )
             kernel, there's no point in flushing them back to the
             guest state.  Indeed doing so could be construed as
             incorrect. */
-
          SysRes sres 
-            = VG_(do_syscall6)(sysno, sci->args.arg1, sci->args.arg2, 
-                                      sci->args.arg3, sci->args.arg4, 
-                                      sci->args.arg5, sci->args.arg6 );
+            = VG_(do_syscall)(sysno, sci->args.arg1, sci->args.arg2, 
+                                     sci->args.arg3, sci->args.arg4, 
+                                     sci->args.arg5, sci->args.arg6,
+                                     sci->args.arg7, sci->args.arg8 );
          sci->status = convert_SysRes_to_SyscallStatus(sres);
 
          PRINT("[sync] --> %s(0x%llx)\n",
-               sci->status.what==SsSuccess ? "Success" : "Failure",
-               (Long)sci->status.val );
+               sci->status.sres.isError ? "Failure" : "Success",
+               sci->status.sres.isError ? (ULong)sci->status.sres.err
+                                        : (ULong)sci->status.sres.res );
       }
    }
 
-   vg_assert(sci->status.what == SsFailure 
-             || sci->status.what == SsSuccess);
+   vg_assert(sci->status.what == SsComplete);
 
    vg_assert(VG_(is_running_thread)(tid));
 
    /* Dump the syscall result back in the guest state.  This is
       a platform-specific action. */
-   putSyscallStatusIntoGuestState( &sci->status, &tst->arch.vex );
+   if (!(sci->flags & SfNoWriteResult))
+      putSyscallStatusIntoGuestState( &sci->status, &tst->arch.vex );
 
    /* Situation now:
       - the guest state is now correctly modified following the syscall
@@ -899,12 +1129,13 @@ void VG_(post_syscall) (ThreadId tid)
 
    /* Validate current syscallInfo entry.  In particular we require
       that the current .status matches what's actually in the guest
-      state. */
-   vg_assert(sci->status.what == SsFailure 
-             || sci->status.what == SsSuccess);
+      state.  At least in the normal case where we have actually
+      previously written the result into the guest state. */
+   vg_assert(sci->status.what == SsComplete);
 
    getSyscallStatusFromGuestState( &test_status, &tst->arch.vex );
-   vg_assert(eq_SyscallStatus( &sci->status, &test_status ));
+   if (!(sci->flags & SfNoWriteResult))
+      vg_assert(eq_SyscallStatus( &sci->status, &test_status ));
    /* Ok, looks sane */
 
    /* Get the system call number.  Because the pre-handler isn't
@@ -922,13 +1153,14 @@ void VG_(post_syscall) (ThreadId tid)
    VG_TRACK( post_reg_write, Vg_CoreSysCall, tid, layout.o_retval, 
                                                   sizeof(UWord) );
 
-   /* Consider, either success or failure.  Now run the post handler if:
+   /* pre: status == Complete (asserted above) */
+   /* Consider either success or failure.  Now run the post handler if:
       - it exists, and
-      - status==Success or (status==Fail and PostOnFail is set)
+      - Success or (Failure and PostOnFail is set)
    */
    if (ent->after
-       && (sci->status.what == SsSuccess
-           || (sci->status.what == SsFailure
+       && ((!sci->status.sres.isError)
+           || (sci->status.sres.isError
                && (sci->flags & SfPostOnFail) ))) {
 
       (ent->after)( tid, &sci->args, &sci->status );
@@ -938,17 +1170,15 @@ void VG_(post_syscall) (ThreadId tid)
       post-handler for sys_open can change the result from success to
       failure if the kernel supplied a fd that it doesn't like), once
       again dump the syscall result back in the guest state.*/
-   putSyscallStatusIntoGuestState( &sci->status, &tst->arch.vex );
+   if (!(sci->flags & SfNoWriteResult))
+      putSyscallStatusIntoGuestState( &sci->status, &tst->arch.vex );
 
    /* Do any post-syscall actions required by the tool. */
-   if (VG_(needs).syscall_wrapper) {
-      SysRes res;
-      res.val     = sci->status.val;
-      res.isError = sci->status.what == SsFailure;
-      VG_TDICT_CALL(tool_post_syscall, tid, sysno, res);
-   }
+   if (VG_(needs).syscall_wrapper)
+      VG_TDICT_CALL(tool_post_syscall, tid, sysno, sci->status.sres);
 
    /* The syscall is done. */
+   vg_assert(sci->status.what == SsComplete);
    sci->status.what = SsIdle;
 
    /* The pre/post wrappers may have concluded that pending signals
@@ -1017,7 +1247,7 @@ void ML_(fixup_guest_state_to_restart_syscall) ( ThreadArchState* arch )
       
       if (p[0] != 0xcd || p[1] != 0x80)
          VG_(message)(Vg_DebugMsg,
-                      "?! restarting over syscall at %p %02x %02x\n",
+                      "?! restarting over syscall at %#x %02x %02x\n",
                       arch->vex.guest_EIP, p[0], p[1]); 
 
       vg_assert(p[0] == 0xcd && p[1] == 0x80);
@@ -1036,7 +1266,7 @@ void ML_(fixup_guest_state_to_restart_syscall) ( ThreadArchState* arch )
       
       if (p[0] != 0x0F || p[1] != 0x05)
          VG_(message)(Vg_DebugMsg,
-                      "?! restarting over syscall at %p %02x %02x\n",
+                      "?! restarting over syscall at %#llx %02x %02x\n",
                       arch->vex.guest_RIP, p[0], p[1]); 
 
       vg_assert(p[0] == 0x0F && p[1] == 0x05);
@@ -1055,8 +1285,34 @@ void ML_(fixup_guest_state_to_restart_syscall) ( ThreadArchState* arch )
 
       if (p[0] != 0x44 || p[1] != 0x0 || p[2] != 0x0 || p[3] != 0x02)
          VG_(message)(Vg_DebugMsg,
-                      "?! restarting over syscall at %p %02x %02x %02x %02x\n",
-                      arch->vex.guest_CIA, p[0], p[1], p[2], p[3]);
+                      "?! restarting over syscall at %#llx %02x %02x %02x %02x\n",
+                      arch->vex.guest_CIA + 0ULL, p[0], p[1], p[2], p[3]);
+
+      vg_assert(p[0] == 0x44 && p[1] == 0x0 && p[2] == 0x0 && p[3] == 0x2);
+   }
+
+#elif defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
+   /* Hmm.  This is problematic, because on AIX the kernel resumes
+      after a syscall at LR, not at the insn following SC.  Hence
+      there is no obvious way to figure out where the SC is.  Current
+      solution is to have a pseudo-register in the guest state,
+      CIA_AT_SC, which holds the address of the most recent SC
+      executed.  Backing up to that syscall then simply involves
+      copying that value back into CIA (the program counter). */
+   arch->vex.guest_CIA = arch->vex.guest_CIA_AT_SC;
+
+   /* Make sure our caller is actually sane, and we're really backing
+      back over a syscall.
+
+      sc == 44 00 00 02
+   */
+   {
+      UChar *p = (UChar *)arch->vex.guest_CIA;
+
+      if (p[0] != 0x44 || p[1] != 0x0 || p[2] != 0x0 || p[3] != 0x02)
+         VG_(message)(Vg_DebugMsg,
+                      "?! restarting over syscall at %#lx %02x %02x %02x %02x\n",
+                      (UWord)arch->vex.guest_CIA, p[0], p[1], p[2], p[3]);
 
       vg_assert(p[0] == 0x44 && p[1] == 0x0 && p[2] == 0x0 && p[3] == 0x2);
    }
@@ -1100,7 +1356,7 @@ void
 VG_(fixup_guest_state_after_syscall_interrupted)( ThreadId tid, 
                                                   Addr     ip, 
                                                   UWord    sysnum, 
-                                                  SysRes   sysret,
+                                                  SysRes   sres,
                                                   Bool     restart)
 {
    /* Note that the sysnum arg seems to contain not-dependable-on info
@@ -1122,8 +1378,8 @@ VG_(fixup_guest_state_after_syscall_interrupted)( ThreadId tid,
                    (Int)tid,
                    (ULong)ip, 
                    restart ? "True" : "False", 
-                   sysret.isError ? "True" : "False",
-                   (Long)(Word)sysret.val );
+                   sres.isError ? "True" : "False",
+                   (Long)(Word)(sres.isError ? sres.err : sres.res) );
 
    vg_assert(VG_(is_valid_tid)(tid));
    vg_assert(tid >= 1 && tid < VG_N_THREADS);
@@ -1137,7 +1393,7 @@ VG_(fixup_guest_state_after_syscall_interrupted)( ThreadId tid,
       (real) IP at the time of the signal, and act accordingly. */
 
    if (ip < ML_(blksys_setup) || ip >= ML_(blksys_finished)) {
-      VG_(printf)("  not in syscall (%p - %p)\n", 
+      VG_(printf)("  not in syscall (%#lx - %#lx)\n",
                   ML_(blksys_setup), ML_(blksys_finished));
       /* Looks like we weren't in a syscall at all.  Hmm. */
       vg_assert(sci->status.what != SsIdle);
@@ -1168,7 +1424,8 @@ VG_(fixup_guest_state_after_syscall_interrupted)( ThreadId tid,
          canonical = convert_SysRes_to_SyscallStatus( 
                         VG_(mk_SysRes_Error)( VKI_EINTR ) 
                      );
-         putSyscallStatusIntoGuestState( &canonical, &th_regs->vex );
+         if (!(sci->flags & SfNoWriteResult))
+            putSyscallStatusIntoGuestState( &canonical, &th_regs->vex );
          sci->status = canonical;
          VG_(post_syscall)(tid);
       }
@@ -1180,9 +1437,10 @@ VG_(fixup_guest_state_after_syscall_interrupted)( ThreadId tid,
          Write the SysRes we were supplied with back to the guest
          state. */
       if (debug)
-         VG_(printf)("  completed\n", sysret);
-      canonical = convert_SysRes_to_SyscallStatus( sysret );
-      putSyscallStatusIntoGuestState( &canonical, &th_regs->vex );
+         VG_(printf)("  completed\n");
+      canonical = convert_SysRes_to_SyscallStatus( sres );
+      if (!(sci->flags & SfNoWriteResult))
+         putSyscallStatusIntoGuestState( &canonical, &th_regs->vex );
       sci->status = canonical;
       VG_(post_syscall)(tid);
    } 
