@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward 
+   Copyright (C) 2000-2008 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -29,6 +29,7 @@
 */
 
 #include "pub_core_basics.h"
+#include "pub_core_vki.h"
 #include "pub_core_threadstate.h"
 #include "pub_core_libcassert.h"
 
@@ -110,14 +111,28 @@ Int VG_(count_living_threads)(void)
    return count;
 }
 
+/* Return the number of threads in VgTs_Runnable state */
+Int VG_(count_runnable_threads)(void)
+{
+   Int count = 0;
+   ThreadId tid;
+
+   for(tid = 1; tid < VG_N_THREADS; tid++)
+      if (VG_(threads)[tid].status == VgTs_Runnable)
+	 count++;
+
+   return count;
+}
+
 /* Given an LWP id (ie, real kernel thread id), find the corresponding
    ThreadId */
-ThreadId VG_(get_lwp_tid)(Int lwp)
+ThreadId VG_(lwpid_to_vgtid)(Int lwp)
 {
    ThreadId tid;
    
    for(tid = 1; tid < VG_N_THREADS; tid++)
-      if (VG_(threads)[tid].status != VgTs_Empty && VG_(threads)[tid].os_state.lwpid == lwp)
+      if (VG_(threads)[tid].status != VgTs_Empty 
+          && VG_(threads)[tid].os_state.lwpid == lwp)
 	 return tid;
 
    return VG_INVALID_THREADID;
