@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward 
+   Copyright (C) 2000-2008 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -52,13 +52,9 @@
 // For varargs types
 #include <stdarg.h>
 
-// Kernel types.  Might as well have them here, they're used so broadly
-// (eg. in pub_core_threadstate.h).
-#if defined(VGO_linux)
-#  include "vki-linux.h"
-#else
-#  error Unknown OS
-#endif
+/* For HAVE_BUILTIN_EXPECT */
+#include "config.h"
+
 
 /* ---------------------------------------------------------------------
    builtin types
@@ -87,6 +83,10 @@ typedef ULong                 Off64T;     // 64             64
 #  define NULL ((void*)0)
 #endif
 
+/* This is just too useful to not have around the place somewhere. */
+typedef  struct { UWord uw1; UWord uw2; }  UWordPair;
+
+
 /* ---------------------------------------------------------------------
    non-builtin types
    ------------------------------------------------------------------ */
@@ -98,14 +98,34 @@ typedef ULong                 Off64T;     // 64             64
 typedef UInt ThreadId;
 
 /* An abstraction of syscall return values.
-   When .isError == False, val holds the return value.
-   When .isError == True,  val holds the error code.
+   Linux:
+      When .isError == False, 
+         res holds the return value, and err is zero.
+      When .isError == True,  
+         err holds the error code, and res is zero.
+
+   AIX:
+      res is the POSIX result of the syscall.
+      err is the corresponding errno value.
+      isError === err==0
+
+      Unlike on Linux, it is possible for 'err' to be nonzero (thus an
+      error has occurred), nevertheless 'res' is also nonzero.  AIX
+      userspace does not appear to consistently inspect 'err' to
+      determine whether or not an error has occurred.  For example,
+      sys_open() will return -1 for 'val' if a file cannot be opened,
+      as well as the relevant errno value in 'err', but AIX userspace
+      then consults 'val' to figure out if the syscall failed, rather
+      than looking at 'err'.  Hence we need to represent them both.
 */
-typedef struct { 
-   UWord val;
-   Bool  isError;
-}
-SysRes;
+typedef
+   struct {
+      UWord res;
+      UWord err;
+      Bool  isError;
+   }
+   SysRes;
+
 
 /* ---------------------------------------------------------------------
    Miscellaneous (word size, endianness, regparmness, stringification)
@@ -141,6 +161,16 @@ SysRes;
 // Where to send bug reports to.
 #define VG_BUGS_TO "www.valgrind.org"
 
+/* Branch prediction hints. */
+#if HAVE_BUILTIN_EXPECT
+#  define LIKELY(x)   __builtin_expect(!!(x), 1)
+#  define UNLIKELY(x) __builtin_expect((x), 0)
+#else
+#  define LIKELY(x)   (x)
+#  define UNLIKELY(x) (x)
+#endif
+
+
 #endif /* __PUB_TOOL_BASICS_H */
 
 /*--------------------------------------------------------------------*/
