@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2007 Julian Seward
+   Copyright (C) 2000-2008 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -37,6 +37,32 @@
 
 /* To use this file you must first include pub_tool_vki.h. */
 
+/* Note that VG_(stat) and VG_(fstat) write to a 'struct vg_stat*' and
+   not a 'struct vki_stat*' or a 'struct vki_stat64*'.  'struct
+   vg_stat*' is not the same as either of the vki_ versions.  No
+   specific vki_stat{,64} kernel structure will work and is
+   consistently available on different architectures on Linux, so we
+   have to use this 'struct vg_stat' impedance-matching type
+   instead. */
+struct vg_stat {
+   ULong   st_dev;
+   ULong   st_ino;
+   ULong   st_nlink;
+   UInt    st_mode;
+   UInt    st_uid;
+   UInt    st_gid;
+   ULong   st_rdev;
+   Long    st_size;
+   ULong   st_blksize;
+   ULong   st_blocks;
+   ULong   st_atime;
+   ULong   st_atime_nsec;
+   ULong   st_mtime;
+   ULong   st_mtime_nsec;
+   ULong   st_ctime;
+   ULong   st_ctime_nsec;
+};
+
 extern SysRes VG_(open)   ( const Char* pathname, Int flags, Int mode );
 extern void   VG_(close)  ( Int fd );
 extern Int    VG_(read)   ( Int fd, void* buf, Int count);
@@ -44,9 +70,10 @@ extern Int    VG_(write)  ( Int fd, const void* buf, Int count);
 extern Int    VG_(pipe)   ( Int fd[2] );
 extern OffT   VG_(lseek)  ( Int fd, OffT offset, Int whence );
 
-extern SysRes VG_(stat)   ( Char* file_name, struct vki_stat* buf );
-extern Int    VG_(fstat)  ( Int   fd,        struct vki_stat* buf );
+extern SysRes VG_(stat)   ( Char* file_name, struct vg_stat* buf );
+extern Int    VG_(fstat)  ( Int   fd,        struct vg_stat* buf );
 extern SysRes VG_(dup)    ( Int oldfd );
+extern SysRes VG_(dup2)   ( Int oldfd, Int newfd );
 extern Int    VG_(rename) ( Char* old_name, Char* new_name );
 extern Int    VG_(unlink) ( Char* file_name );
 
